Package: OpenLand
Title: Quantitative Analysis and Visualization of LUCC
Version: 1.0.5
Authors@R: c(
    person(given = "Reginal",
           family = "Exavier",
           role = c("aut", "cre"),
           email = "reginalexavier@rocketmail.com",
           comment = c(ORCID = "0000-0002-5237-523X")),
    person(given = "Peter",
           family = "Zeilhofer",
           role = "aut",
           email = "zeilhoferpeter@gmail.com"))
Description: Tools for the analysis of land use and cover (LUC) time series. It 
    includes support for loading spatiotemporal raster data and synthesized 
    spatial plotting. Several LUC change (LUCC) metrics in regular or irregular 
    time intervals can be extracted and visualized through one- and multistep 
    sankey and chord diagrams. A complete intensity analysis according to 
    Aldwaik and Pontius (2012) <doi:10.1016/j.landurbplan.2012.02.010> is 
    implemented, including tools for the generation of standardized multilevel 
    output graphics.
License: GPL-3
URL: https://reginalexavier.github.io/OpenLand/,
        https://github.com/reginalexavier/OpenLand
BugReports: https://github.com/reginalexavier/OpenLand/issues
Encoding: UTF-8
LazyData: true
Depends: R (>= 3.4.0)
Imports: dplyr (>= 0.8.3), tidyr (>= 1.0.0), ggplot2 (>= 3.2.1),
        gridExtra (>= 2.3), grid, circlize (>= 0.4.8), networkD3 (>=
        0.4), raster (>= 3.0.7), methods
Collate: 'OpenLand-package.R' 'rasters_input.R' 'demolandscape.R'
        'contingencyTable.R' 'data.R' 'generalfunctions.R'
        'intensityClass.R' 'generic_method.R' 'intensityAnalysis.R'
        'plotMethods.R' 'otherplots.R'
Suggests: tmap, knitr, rmarkdown, covr, testthat, spelling
VignetteBuilder: knitr
Language: en-US
Config/roxygen2/version: 8.0.0
NeedsCompilation: no
Packaged: 2026-07-10 02:08:38 UTC; tredgi
Author: Reginal Exavier [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-5237-523X>),
  Peter Zeilhofer [aut]
Maintainer: Reginal Exavier <reginalexavier@rocketmail.com>
Repository: CRAN
Date/Publication: 2026-07-10 06:20:08 UTC

# Generated by roxygen2: do not edit by hand

export(.demo_landscape)
export(acc_changes)
export(barplotLand)
export(chordDiagramLand)
export(contingencyTable)
export(intensityAnalysis)
export(netgrossplot)
export(plot)
export(sankeyLand)
export(summary_dir)
export(summary_map)
exportClasses(Category)
exportClasses(Interval)
exportClasses(Transition)
exportMethods(.input_rasters)
exportMethods(plot)
import(dplyr)
import(ggplot2)
import(methods)
importFrom(graphics,legend)
importFrom(graphics,par)
importFrom(graphics,text)
importFrom(grid,gpar)
importFrom(grid,textGrob)
importFrom(gridExtra,grid.arrange)
importFrom(raster,brick)
importFrom(raster,compareRaster)
importFrom(raster,crosstab)
importFrom(raster,overlay)
importFrom(raster,raster)
importFrom(raster,stack)
importFrom(raster,unstack)
importFrom(raster,values)

<!-- README.md is generated from README.Rmd. Please edit that file -->

# OpenLand <img src='man/figures/logo.png' align="right" height="139" />

<!-- badges: start -->

[![R-CMD-check](https://github.com/reginalexavier/OpenLand/workflows/R-CMD-check/badge.svg)](https://github.com/reginalexavier/OpenLand/actions)
[![codecov](https://codecov.io/gh/reginalexavier/OpenLand/branch/master/graph/badge.svg?token=9wGWKYH0X8)](https://app.codecov.io/gh/reginalexavier/OpenLand)
[![License: GPL
v3](https://img.shields.io/badge/License-GPLv3-blue.svg)](https://www.gnu.org/licenses/gpl-3.0)
[![CRAN
status](https://www.r-pkg.org/badges/version/OpenLand)](https://CRAN.R-project.org/package=OpenLand)
<!-- badges: end -->

OpenLand is an open-source R package for the analysis of land use and
cover (LUC) time series. It includes support for consistency check and
loading spatiotemporal raster data and synthesized spatial plotting.
Several LUC change (LUCC) metrics in regular or irregular time intervals
can be extracted and visualized through one- and multistep sankey and
chord diagrams. A complete intensity analysis according to (Aldwaik and
Pontius 2012) is implemented, including tools for the generation of
standardized multilevel output graphics.

## Installation

Install the released version of OpenLand from CRAN:

``` r
install.packages("OpenLand")
```

Or install the development version from GitHub with:

``` r
# install.packages("devtools")
devtools::install_github("reginalexavier/OpenLand")
```

## Illustrative Example

This is a basic example which shows how OpenLand works, for a more
detailed illustration, please see our
[vignettes](https://reginalexavier.github.io/OpenLand/articles/openland_vignette.html).

The OpenLand functionality is illustrated for a LUC dataset of São
Lourenço river basin, a major Pantanal wetland contribution area as
provided by the 4<sup>th</sup> edition of the Monitoring of Changes in
Land cover and Land Use in the Upper Paraguay River Basin - Brazilian
portion - Review Period: 2012 to 2014 (Embrapa Pantanal, Instituto SOS
Pantanal, and WWF-Brasil 2015). The time series is composed by five LUC
maps (2002, 2008, 2010, 2012 and 2014). The study area of approximately
22,400 km<sup>2</sup> is located in the Cerrado Savannah biom in the
southeast of the Brazilian state of Mato Grosso. For processing in the
OpenLand package, the original multi-year shape file was transformed
into rasters and then saved as a 5-layer `RasterStack`
(`SaoLourencoBasin`), available from a public repository
[(10.5281/zenodo.3685229)](https://doi.org/10.5281/zenodo.3685230) as an
`.RDA` file which can be loaded into `R`.

``` r
# Loading the package
library(OpenLand)
```

##### **What is Intensity Analysis?**

Intensity Analysis (IA) is a quantitative method to analyze LUC maps at
several time steps, using cross-tabulation matrices, where each matrix
summarizes the LUC change at each time interval. IA evaluates in three
levels the deviation between observed change intensity and hypothesized
uniform change intensity. Hereby, each level details information given
by the previous analysis level. First, the **interval level** indicates
how size and rate of change varies across time intervals. Second, the
**category level** examines for each time interval how the size and
intensity of gross losses and gross gains in each category vary across
categories for each time interval. Third, the **transition level**
determines for each category how the size and intensity of a category’s
transitions vary across the other categories that are available for that
transition. At each level, the method tests for stationarity of patterns
across time intervals (Aldwaik and Pontius 2012).

### Outcomes of intensity analysis

The data is extracted from the rasters with the
[`contingencyTable()`](https://reginalexavier.github.io/OpenLand/reference/contingencyTable.html)
function which returns a multiple grid information in tables for the
next processing steps. Within the OpenLand package, the
[`intensityAnalysis()`](https://reginalexavier.github.io/OpenLand/reference/intensityAnalysis.html)
function computes the three levels of analysis. It requires the object
returned by the `contingenceTable()` function and that the user
predefines two LUC categories `n` and `m`. Generally, `n` is a target
category which experienced relevant gains and `m` a category with
important losses.

``` r
my_test <- intensityAnalysis(
  dataset = SL_2002_2014, # here the outcome from the `contingenceTable()` function
  category_n = "Ap", category_m = "SG"
)

# it returns a list with 6 objects
names(my_test)
#> [1] "lulc_table"           "interval_lvl"         "category_lvlGain"    
#> [4] "category_lvlLoss"     "transition_lvlGain_n" "transition_lvlLoss_m"
```

The `intensityAnalysis()` function returns 6 objects: lulc_table,
interval_lvl, category_lvlGain, category_lvlLoss, transition_lvlGain_n,
transition_lvlLoss_m. Here, we adopted an object-oriented approach that
allows to set specific methods for plotting the intensity objects.
Specifically, we used the S4 class, which requires the formal definition
of classes and methods (Chambers 2008).

#### Presentation of an intensity object

In this example we will show an object from `Category` class. A
`Category` object contains three slots: the first contains the colors
associated with the legend items as name attributes, the second slot
contains a table of the **category level** result *(gain
(G<sub>tj</sub>) or loss (L<sub>ti</sub>) values)* and the third slot
contains a table storing the results of a stationarity test.

``` r
my_test$category_lvlGain
#> An object of class "Category"
#> Slot "lookupcolor":
#>        Ap        FF        SA        SG        aa        SF      Agua        Iu 
#> "#FFE4B5" "#228B22" "#00FF00" "#CAFF70" "#EE6363" "#00CD00" "#436EEE" "#FFAEB9" 
#>        Ac         R        Im 
#> "#FFA54F" "#68228B" "#636363" 
#> 
#> Slot "categoryData":
#> # A tibble: 23 × 6
#> # Groups:   Period, To [23]
#>    Period    To    Interval  GG_km2   Gtj    St
#>    <fct>     <fct>    <int>   <dbl> <dbl> <dbl>
#>  1 2012-2014 aa           2  14.9   0.510  1.66
#>  2 2012-2014 Ap           2 612.    3.92   1.66
#>  3 2012-2014 Ac           2 110.    1.14   1.66
#>  4 2012-2014 Im           2   0.195 0.337  1.66
#>  5 2012-2014 Iu           2   6.79  2.67   1.66
#>  6 2010-2012 aa           2  47.0   1.18   2.12
#>  7 2010-2012 Ap           2 707.    4.84   2.12
#>  8 2010-2012 Ac           2 189.    2.00   2.12
#>  9 2010-2012 Iu           2   1.90  0.792  2.12
#> 10 2010-2012 R            2   2.76  0.951  2.12
#> # ℹ 13 more rows
#> 
#> Slot "categoryStationarity":
#> # A tibble: 12 × 5
#>    To     Gain     N Stationarity Test 
#>    <fct> <int> <int> <chr>        <chr>
#>  1 aa        2     4 Active Gain  N    
#>  2 Ap        2     4 Active Gain  N    
#>  3 Ac        1     4 Active Gain  N    
#>  4 Iu        2     4 Active Gain  N    
#>  5 Agua      1     4 Active Gain  N    
#>  6 R         2     4 Active Gain  N    
#>  7 aa        2     4 Dormant Gain N    
#>  8 Ap        2     4 Dormant Gain N    
#>  9 Ac        3     4 Dormant Gain N    
#> 10 Im        3     4 Dormant Gain N    
#> 11 Iu        2     4 Dormant Gain N    
#> 12 R         1     4 Dormant Gain N
```

#### Plotting an intensity object

Visualizations of the IA results are obtained from the
`plot(intensity-object)` function. For more details on the function
arguments, please see the documentation of the
[`plot()`](https://reginalexavier.github.io/OpenLand/reference/plot.html)
method.

``` r
plot(my_test$category_lvlGain,
  labels = c(
    leftlabel = bquote("Gain Area (" ~ km^2 ~ ")"),
    rightlabel = "Intensity Gain (%)"
  ),
  marginplot = c(.3, .3), labs = c("Categories", "Uniform intensity"),
  leg_curv = c(x = 1, y = .5),
  fontsize_ui = 8
)
```

<div class="figure" style="text-align: center">

<img src="man/figures/README-cat_level-1.png" alt="Gain area outcome - Category level" width="80%" />
<p class="caption">
Gain area outcome - Category level
</p>

</div>

### Miscellaneous visualization tools

OpenLand provides a bench of visualization tools of LUCC metrics.
One-step transitions can be balanced by net and gross changes of all
categories through a combined bar chart. Transitions between LUC
categories can be detailed by a circular chord chart, based on the
Circlize package (Gu et al. 2014). An implementation of Sankey diagram
based on the networkD3 package (Allaire et al. 2017) allow the
representation of one- and multistep LUCC between categories. Areal
development of all LUC categories throughout the observation period can
be visualized by a grouped bar chart.

##### Net and Gross gain and loss

``` r
netgrossplot(
  dataset = SL_2002_2014$lulc_Multistep,
  legendtable = SL_2002_2014$tb_legend,
  xlab = "LUC Category",
  ylab = bquote("Area (" ~ km^2 ~ ")"),
  changesLabel = c(GC = "Gross changes", NG = "Net Gain", NL = "Net Loss"),
  color = c(GC = "gray70", NG = "#006400", NL = "#EE2C2C")
)
```

<div class="figure" style="text-align: center">

<img src="man/figures/README-ng_plot-1.png" alt="Net Gross Changes 2002 - 2014" width="80%" />
<p class="caption">
Net Gross Changes 2002 - 2014
</p>

</div>

##### Chord Diagram (2002 - 2014)

``` r
chordDiagramLand(
  dataset = SL_2002_2014$lulc_Onestep,
  legendtable = SL_2002_2014$tb_legend
)
```

<div class="figure" style="text-align: center">

<img src="man/figures/README-chordDiagram-1.png" alt="Chord Diagram 2002 - 2014 (area in km^2^)" width="100%" />
<p class="caption">
Chord Diagram 2002 - 2014 (area in km<sup>2</sup>)
</p>

</div>

##### Sankey Multi Step (2002, 2008, 2010, 2012, 2014)

``` r
# sankeyLand(dataset = SL_2002_2014$lulc_Multistep,
#            legendtable = SL_2002_2014$tb_legend)
```

<img src="man/figures/sankey_multi.png" alt="" width="80%" style="display: block; margin: auto;" />

#### Other functions

OpenLand enables furthermore the spatial screening of LUCC frequencies
for one or a series of raster layers with
[`summary_map()`](https://reginalexavier.github.io/OpenLand/reference/summary_map.html)
and
[`summary_dir()`](https://reginalexavier.github.io/OpenLand/reference/summary_dir.html).
The
[`acc_changes()`](https://reginalexavier.github.io/OpenLand/reference/acc_changes.html)
function returns for a LUC time series the number of times a pixel has
changed during the analysed period, returning a grid layer and a table
with the percentages of transition numbers in the study area. Here we
use the [tmap](https://github.com/r-tmap/tmap) package for plotting the
outcomes of the `acc_changes()` function.

<div class="figure" style="text-align: center">

<img src="man/figures/acc_mymap.png" alt="Accumulated changes in pixels in the interval 2002 - 2014 at four time points (2002, 2008, 2010, 2012, 2014)" width="90%" />
<p class="caption">
Accumulated changes in pixels in the interval 2002 - 2014 at four time
points (2002, 2008, 2010, 2012, 2014)
</p>

</div>

## References

Aldwaik, Safaa Zakaria, and Robert Gilmore Pontius. 2012. “Intensity
analysis to unify measurements of size and stationarity of land changes
by interval, category, and transition.” Landsc. Urban Plan. 106 (1):
103–14. <https://doi.org/10.1016/j.landurbplan.2012.02.010>.

Allaire, J J, Christopher Gandrud, Kenton Russell, and C J Yetman. 2017.
“networkD3: D3 JavaScript Network Graphs from R.”
<https://cran.r-project.org/package=networkD3>.

Chambers, John. 2008. Software for Data Analysis. Statistics and
Computing. New York, NY: Springer New York.
<https://doi.org/10.1007/978-0-387-75936-4>.

Embrapa Pantanal, Instituto SOS Pantanal, and WWF-Brasil. 2015.
“Mapeamento da Bacia do Alto Paraguai.”

Gu, Zuguang, Lei Gu, Roland Eils, Matthias Schlesner, and Benedikt
Brors. 2014. “circlize implements and enhances circular visualization in
R.” Bioinformatics 30 (19): 2811–2.

------------------------------------------------------------------------

### CITATION:

Reginal Exavier and Peter Zeilhofer. OpenLand: Software for Quantitative
Analysis and Visualization of Land Use and Cover Change. The R Journal,
v. 12, n. 2, p. 359–371, 2021.
[https://doi.org/10.32614/RJ-2021-021](https://journal.r-project.org/articles/RJ-2021-021/index.html).

## ----setup, include = FALSE---------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)

## ----include=FALSE------------------------------------------------------------
url <- "https://zenodo.org/record/3685230/files/SaoLourencoBasin.rda?download=1"
dataset_available <- OpenLand:::.openland_try_download_and_load_rda(url,
  object = "SaoLourencoBasin",
  timeout = 10,
  quiet = TRUE
)

## ----include=FALSE, eval=isFALSE(dataset_available)---------------------------
# knitr::knit_exit(paste0(
#   "## The complete rendering of this document depends on the availability of the dataset in zenodo. ",
#   "Please try at another time."
# ))

## ----message=FALSE, warning=FALSE---------------------------------------------
# first we load the OpenLand package
library(OpenLand)

# The SaoLourencoBasin dataset
SaoLourencoBasin

## ----eval=FALSE, fig.height=5, fig.width=7, include=FALSE---------------------
# # sp::spplot(SaoLourencoBasin)

## -----------------------------------------------------------------------------
SL_2002_2014$lulc_Multistep[1:10, ]

SL_2002_2014$lulc_Onestep[1:10, ]

SL_2002_2014$tb_legend

SL_2002_2014$totalArea

SL_2002_2014$totalInterval

## ----echo=TRUE----------------------------------------------------------------
## editing the category name
SL_2002_2014$tb_legend$categoryName <- factor(
  c(
    "Ap", "FF", "SA", "SG", "aa", "SF",
    "Agua", "Iu", "Ac", "R", "Im"
  ),
  levels = c(
    "FF", "SF", "SA", "SG", "aa", "Ap",
    "Ac", "Im", "Iu", "Agua", "R"
  )
)

## add the color by the same order of the legend,
## it can be the color name (eg. "black") or the HEX value (eg. #000000)
SL_2002_2014$tb_legend$color <- c(
  "#FFE4B5", "#228B22", "#00FF00", "#CAFF70",
  "#EE6363", "#00CD00", "#436EEE", "#FFAEB9",
  "#FFA54F", "#68228B", "#636363"
)

## now we have
SL_2002_2014$tb_legend

## ----eval=FALSE, include=FALSE------------------------------------------------
# knitr::kable(
#   SL_2002_2014$lulc_Multistep[1:10, ]
# )

## -----------------------------------------------------------------------------
testSL <- intensityAnalysis(
  dataset = SL_2002_2014,
  category_n = "Ap", category_m = "SG"
)

# it returns a list with 6 objects
names(testSL)

## ----eval=FALSE, include=FALSE------------------------------------------------
# # showing the objects from the intensity analysis for our illustrative case
# testSL

## ----fig.width=7, fig.height=4------------------------------------------------
plot(testSL$interval_lvl,
  labels = c(
    leftlabel = "Interval Change Area (%)",
    rightlabel = "Annual Change Area (%)"
  ),
  marginplot = c(-8, 0), labs = c("Changes", "Uniform Rate"),
  leg_curv = c(x = 2 / 10, y = 3 / 10)
)

## ----fig.width=7, fig.height=4------------------------------------------------
plot(testSL$category_lvlGain,
  labels = c(
    leftlabel = bquote("Gain Area (" ~ km^2 ~ ")"),
    rightlabel = "Intensity Gain (%)"
  ),
  marginplot = c(.3, .3), labs = c("Categories", "Uniform Rate"),
  leg_curv = c(x = 5 / 10, y = 5 / 10)
)

## ----fig.width=7, fig.height=4------------------------------------------------
plot(testSL$category_lvlLoss,
  labels = c(
    leftlabel = bquote("Loss Area (" ~ km^2 ~ ")"),
    rightlabel = "Loss Intensity (%)"
  ),
  marginplot = c(.3, .3), labs = c("Categories", "Uniform Rate"),
  leg_curv = c(x = 5 / 10, y = 5 / 10)
)

## ----fig.width=7, fig.height=4------------------------------------------------
plot(testSL$transition_lvlGain_n,
  labels = c(
    leftlabel = bquote("Gain of Ap (" ~ km^2 ~ ")"),
    rightlabel = "Intensity Gain of Ap (%)"
  ),
  marginplot = c(.3, .3), labs = c("Categories", "Uniform Rate"),
  leg_curv = c(x = 5 / 10, y = 5 / 10)
)

## ----fig.width=7, fig.height=4------------------------------------------------
plot(testSL$transition_lvlLoss_m,
  labels = c(
    leftlabel = bquote("Loss of SG (" ~ km^2 ~ ")"),
    rightlabel = "Intensity Loss of SG (%)"
  ),
  marginplot = c(.3, .3), labs = c("Categories", "Uniform Rate"),
  leg_curv = c(x = 1 / 10, y = 5 / 10)
)

## ----fig.width=7, fig.height=4------------------------------------------------
netgrossplot(
  dataset = SL_2002_2014$lulc_Multistep,
  legendtable = SL_2002_2014$tb_legend,
  xlab = "LUC Category",
  ylab = bquote("Area (" ~ km^2 ~ ")"),
  changesLabel = c(GC = "Gross changes", NG = "Net Gain", NL = "Net Loss"),
  color = c(GC = "gray70", NG = "#006400", NL = "#EE2C2C")
)

## ----fig.width=7, fig.height=6------------------------------------------------
chordDiagramLand(
  dataset = SL_2002_2014$lulc_Onestep,
  legendtable = SL_2002_2014$tb_legend
)

## ----fig.width=8, fig.height=4------------------------------------------------
sankeyLand(
  dataset = SL_2002_2014$lulc_Multistep,
  legendtable = SL_2002_2014$tb_legend
)

## ----fig.width=8, fig.height=4------------------------------------------------
sankeyLand(
  dataset = SL_2002_2014$lulc_Onestep,
  legendtable = SL_2002_2014$tb_legend
)

## ----fig.width=7, fig.height=4------------------------------------------------
barplotLand(
  dataset = SL_2002_2014$lulc_Multistep,
  legendtable = SL_2002_2014$tb_legend,
  xlab = "Year",
  ylab = bquote("Area (" ~ km^2 ~ ")"),
  area_km2 = TRUE
)

## ----eval=FALSE, include=TRUE-------------------------------------------------
# testacc <- acc_changes(SaoLourencoBasin)
# 
# testacc

## ----eval=FALSE, fig.height=7, fig.width=7, include=TRUE----------------------
# tmap_options(max.raster = c(plot = 41711112, view = 41711112))
# 
# acc_map <- tmap::tm_shape(testacc[[1]]) +
#   tmap::tm_raster(
#     style = "cat",
#     labels = c(
#       paste0(testacc[[2]]$PxValue[1], " Change", " (", round(testacc[[2]]$Percent[1], 2), "%", ")"),
#       paste0(testacc[[2]]$PxValue[2], " Change", " (", round(testacc[[2]]$Percent[2], 2), "%", ")"),
#       paste0(testacc[[2]]$PxValue[3], " Changes", " (", round(testacc[[2]]$Percent[3], 2), "%", ")")
#     ),
#     palette = c("#757575", "#FFD700", "#CD0000"),
#     title = "Changes in the interval \n2002 - 2014"
#   ) +
#   tmap::tm_legend(
#     position = c(0.01, 0.2),
#     legend.title.size = 1.2,
#     legend.title.fontface = "bold",
#     legend.text.size = 0.8
#   ) +
#   tmap::tm_compass(
#     type = "arrow",
#     position = c("right", "top"),
#     size = 3
#   ) +
#   tmap::tm_scale_bar(
#     breaks = c(seq(0, 40, 10)),
#     position = c(0.76, 0.001),
#     text.size = 0.6
#   ) +
#   tmap::tm_credits(
#     paste0(
#       "Case of Study site",
#       "\nAccumulate changes from 2002 to 2014",
#       "\nData create with OpenLand package",
#       "\nLULC derived from Embrapa Pantanal, Instituto SOS Pantanal, and WWF-Brasil 2015."
#     ),
#     size = 0.7,
#     position = c(0.01, -0, 01)
#   ) +
#   tmap::tm_graticules(
#     n.x = 6,
#     n.y = 6,
#     lines = FALSE,
#     # alpha = 0.1
#     labels.rot = c(0, 90)
#   ) +
#   tmap::tm_layout(inner.margins = c(0.02, 0.02, 0.02, 0.02))
# 
# 
# tmap::tmap_save(acc_map,
#   filename = "vignettes/acc_mymap.png",
#   width = 7,
#   height = 7
# )

## ----mymap, echo=FALSE, fig.cap='Accumulated changes in pixels in the interval 2002 - 2014 at four time points (2002, 2008, 2010, 2012, 2014)', out.width='100%'----
knitr::include_graphics("acc_mymap.png")

#' Create Raster with Random pixel Value
#'
#' This function creates a raster series with some setup like the layer name and
#' the sample value for the lulc
#'
#' @param year numeric. A vector of year, first and last included.
#' @param nrows numeric. nrows of the raster to be created.
#' @param ncols numeric. ncols of the raster to be created.
#' @param res numeric. the resolution of the raster to be created.
#' @param xmn numeric. x minimum extent.
#' @param xmx numeric. x maximum extent.
#' @param ymn numeric. y minimum extent.
#' @param ymx numeric. y maximum extent.
#' @param crs character. the coordinate referencing system.
#' @param category A numeric vector of the raster categories.
#' @param prob A numeric vector of the probability of occurrence for the
#' category list.
#'
#'
#' @seealso \code{\link[raster]{raster}}
#'
#' @import dplyr
#' @return list
#' @export
#' @keywords internal
#' @examples
#' .demo_landscape(
#'   year = 2000:2005,
#'   res = 1,
#'   crs = "+proj=utm +zone=21 +south +ellps=GRS80 +units=m +no_defs"
#' )
#'
.demo_landscape <- function(year,
                            nrows = 100,
                            ncols = 100,
                            res = 1,
                            xmn = 0,
                            xmx = 100,
                            ymn = 0,
                            ymx = 100,
                            crs = NA,
                            category = 1:5,
                            prob = NULL) {
  # a sample raster
  landscape <- raster::raster(
    nrows = nrows,
    ncols = ncols,
    xmn = xmn,
    xmx = xmx,
    ymn = ymn,
    ymx = ymx,
    resolution = res,
    crs = crs
  )

  mapdemo <- function(year01, pixvalue) { # an atribuitor of values
    raster::values(landscape) <- pixvalue
    names(landscape) <- paste0("landscape_", year01)
    landscape
  }

  samplerow <- nrow(landscape)
  samplecol <- ncol(landscape)

  pixsample <-
    lapply(year, function(x) {
      base::sample(
        category,
        samplerow * samplecol,
        replace = TRUE,
        prob = prob
      )
    })


  # create a list of n of them
  raster_list <-
    mapply(function(x, y) {
      mapdemo(year01 = x, pixvalue = y)
    }, year, pixsample, USE.NAMES = TRUE)

  names(raster_list) <- vapply(raster_list, function(x) {
    names(x)
  }, FUN.VALUE = character(1L))

  raster_list
}

context("intensityAnalysis")

set.seed(159)
demo_raster <- .demo_landscape(
  year = 2000:2005,
  res = 1,
  crs = "+proj=utm +zone=21 +south +ellps=GRS80 +units=m +no_defs"
)
demo_cont <- contingencyTable(demo_raster, pixelresolution = 1)



test_that("Behavior of the intensityAnalysis", {
  demo_int_km2 <-
    intensityAnalysis(
      dataset = demo_cont,
      category_n = "GUP",
      category_m = "SXQ",
      area_km2 = TRUE
    )

  demo_int_pixel <-
    intensityAnalysis(
      dataset = demo_cont,
      category_n = "GUP",
      category_m = "SXQ",
      area_km2 = FALSE
    )


  expect_visible(intensityAnalysis(
    dataset = demo_cont,
    category_n = "GUP",
    category_m = "SXQ",
    area_km2 = FALSE
  ))

  expect_visible(intensityAnalysis(
    dataset = demo_cont,
    category_n = "GUP",
    category_m = "SXQ",
    area_km2 = TRUE
  ))


  expect_equal(
    demo_int_km2$interval_lvl$intervalData$U,
    demo_int_pixel$interval_lvl$intervalData$U
  )

  expect_equal(demo_int_km2$interval_lvl$intervalData$PercentChange[3], 80.26)


  expect_equal(
    demo_int_km2$category_lvlGain$categoryData$GG_km2,
    demo_int_pixel$category_lvlGain$categoryData$GG_pixel / 1000000
  )
  expect_equal(
    demo_int_km2$category_lvlLoss$categoryData$GL_km2,
    demo_int_pixel$category_lvlLoss$categoryData$GL_pixel / 1000000
  )

  expect_equal(demo_int_pixel$category_lvlGain$categoryData$GG_pixel[8], 1587)
  expect_equal(demo_int_pixel$category_lvlLoss$categoryData$GL_pixel[10], 1684)

  expect_equal(
    demo_int_km2$transition_lvlGain_n$transitionData$T_i2n_km2,
    demo_int_pixel$transition_lvlGain_n$transitionData$T_i2n_pixel / 1000000
  )
  expect_equal(
    demo_int_km2$transition_lvlLoss_m$transitionData$T_m2j_km2,
    demo_int_pixel$transition_lvlLoss_m$transitionData$T_m2j_pixel / 1000000
  )

  expect_equal(demo_int_pixel$transition_lvlGain_n$transitionData$T_i2n_pixel[18], 380)
  expect_equal(demo_int_pixel$transition_lvlLoss_m$transitionData$T_m2j_pixel[15], 382)

  # stationarity

  expect_equal(
    demo_int_km2$category_lvlGain$categoryStationarity$Stationarity,
    demo_int_pixel$category_lvlGain$categoryStationarity$Stationarity
  )
  expect_equal(
    demo_int_km2$category_lvlLoss$categoryStationarity$Stationarity,
    demo_int_pixel$category_lvlLoss$categoryStationarity$Stationarity
  )

  expect_equal(
    demo_int_km2$transition_lvlGain_n$transitionStationarity$Stationarity,
    demo_int_pixel$transition_lvlGain_n$transitionStationarity$Stationarity
  )
  expect_equal(
    demo_int_km2$transition_lvlLoss_m$transitionStationarity$Stationarity,
    demo_int_pixel$transition_lvlLoss_m$transitionStationarity$Stationarity
  )

  # class

  expect_s4_class(demo_int_km2$interval_lvl, "Interval")
  expect_s4_class(demo_int_pixel$interval_lvl, "Interval")

  expect_s4_class(demo_int_km2$category_lvlGain, "Category")
  expect_s4_class(demo_int_pixel$category_lvlGain, "Category")
  expect_s4_class(demo_int_km2$category_lvlLoss, "Category")
  expect_s4_class(demo_int_pixel$category_lvlLoss, "Category")

  expect_s4_class(demo_int_km2$transition_lvlGain_n, "Transition")
  expect_s4_class(demo_int_pixel$transition_lvlGain_n, "Transition")
  expect_s4_class(demo_int_km2$transition_lvlLoss_m, "Transition")
  expect_s4_class(demo_int_pixel$transition_lvlLoss_m, "Transition")
})

context("intensityClass & generic_method")

set.seed(159)
demo_raster <- .demo_landscape(
  year = 2000:2005,
  res = 1,
  crs = "+proj=utm +zone=21 +south +ellps=GRS80 +units=m +no_defs"
)
demo_cont <- contingencyTable(demo_raster, pixelresolution = 1)

demo_int_pixel <-
  intensityAnalysis(
    dataset = demo_cont,
    category_n = "GUP",
    category_m = "SXQ",
    area_km2 = FALSE
  )


test_that("Behavior of the 3 s4 classes", {
  expect_visible(
    new("Interval", intervalData = demo_int_pixel$interval_lvl$intervalData)
  )
  expect_visible(
    new("Category",
      categoryData = demo_int_pixel$category_lvlGain$categoryData,
      lookupcolor = demo_int_pixel$category_lvlGain$lookupcolor,
      categoryStationarity = demo_int_pixel$category_lvlGain$categoryStationarity
    )
  )
  expect_visible(
    new("Transition",
      transitionData = demo_int_pixel$transition_lvlGain_n$transitionData,
      lookupcolor = demo_int_pixel$transition_lvlGain_n$lookupcolor,
      transitionStationarity = demo_int_pixel$transition_lvlGain_n$transitionStationarity
    )
  )


  expect_s4_class(
    new("Interval", intervalData = demo_int_pixel$interval_lvl$intervalData),
    "Interval"
  )
  expect_s4_class(
    new("Category",
      categoryData = demo_int_pixel$category_lvlGain$categoryData,
      lookupcolor = demo_int_pixel$category_lvlGain$lookupcolor,
      categoryStationarity = demo_int_pixel$category_lvlGain$categoryStationarity
    ),
    "Category"
  )
  expect_s4_class(
    new("Transition",
      transitionData = demo_int_pixel$transition_lvlGain_n$transitionData,
      lookupcolor = demo_int_pixel$transition_lvlGain_n$lookupcolor,
      transitionStationarity = demo_int_pixel$transition_lvlGain_n$transitionStationarity
    ),
    "Transition"
  )
  expect_error(
    new("Interval", intervalData = demo_int_pixel$interval_lvl$intervalData[1:2])
  )
  expect_error(
    new("Category",
      categoryData = demo_int_pixel$category_lvlGain$categoryData[1:4],
      lookupcolor = demo_int_pixel$category_lvlGain$lookupcolor,
      categoryStationarity = demo_int_pixel$category_lvlGain$categoryStationarity
    )
  )
  expect_error(
    new("Transition",
      transitionData = demo_int_pixel$transition_lvlGain_n$transitionData[1:4],
      lookupcolor = demo_int_pixel$transition_lvlGain_n$lookupcolor,
      transitionStationarity = demo_int_pixel$transition_lvlGain_n$transitionStationarity
    )
  )


  expect_error(
    new("Interval", intervalData = as.data.frame(demo_int_pixel$interval_lvl$intervalData))
  )
  expect_error(
    new("Category",
      categoryData = as.data.frame(demo_int_pixel$category_lvlGain$categoryData),
      lookupcolor = demo_int_pixel$category_lvlGain$lookupcolor,
      categoryStationarity = demo_int_pixel$category_lvlGain$categoryStationarity
    )
  )
  expect_error(
    new("Transition",
      transitionData = as.data.frame(demo_int_pixel$transition_lvlGain_n$transitionData),
      lookupcolor = demo_int_pixel$transition_lvlGain_n$lookupcolor,
      transitionStationarity = demo_int_pixel$transition_lvlGain_n$transitionStationarity
    )
  )
})


test_that("Behavior of the acessor", {
  expect_equal(
    demo_int_pixel$interval_lvl@intervalData,
    demo_int_pixel$interval_lvl$intervalData
  )


  expect_equal(
    demo_int_pixel$category_lvlGain@lookupcolor,
    demo_int_pixel$category_lvlGain$lookupcolor
  )

  expect_equal(
    demo_int_pixel$transition_lvlGain_n@lookupcolor,
    demo_int_pixel$transition_lvlGain_n$lookupcolor
  )
})

context("demo_landscape")


test_that("whether the demo gives the same output", {
  set.seed(159)

  demo_raster <- .demo_landscape(2000:2005, res = 1)

  expect_equal(c(class(demo_raster[[1]])), "RasterLayer")

  expect_equal(length(demo_raster), 6)
})

context("contingencyTable")

set.seed(159)
demo_raster <- .demo_landscape(
  year = 2000:2005,
  res = 1,
  crs = "+proj=utm +zone=21 +south +ellps=GRS80 +units=m +no_defs"
)
demo_cont <- contingencyTable(demo_raster, pixelresolution = 1)


test_that("Behavior of the contingencyTable", {
  expect_error(contingencyTable(
    input_raster = c(
      .demo_landscape(year = 2000),
      .demo_landscape(year = 2002, xmx = 50)
    ),
    pixelresolution = 1
  ))

  expect_error(contingencyTable(
    input_raster = .demo_landscape(year = 2000),
    pixelresolution = 1
  ))

  expect_error(contingencyTable(input_raster = 5L))

  expect_error(contingencyTable(
    input_raster = c(
      .demo_landscape(
        year = 2000,
        crs = "+proj=utm +zone=22 +south +ellps=GRS80 +units=m +no_defs"
      ),
      .demo_landscape(
        year = 2002,
        crs = "+proj=utm +zone=21 +south +ellps=GRS80 +units=m +no_defs"
      )
    ),
    pixelresolution = 1
  ))

  expect_silent(contingencyTable(.demo_landscape(2000:2001)))
  expect_silent(contingencyTable(demo_raster, pixelresolution = 1))
  expect_silent(contingencyTable(raster::stack(demo_raster), pixelresolution = 1))
  expect_silent(contingencyTable(raster::brick(demo_raster), pixelresolution = 1))
  expect_length(demo_cont, 5)
  expect_output(str(demo_cont), "List of 5")
  expect_equal(ncol(demo_cont$lulc_Multistep), 8)
  expect_named(demo_cont$lulc_Multistep, colnames(demo_cont$lulc_Onestep))
  expect_equal(
    head(demo_cont$lulc_Multistep$yearFrom, 1),
    head(demo_cont$lulc_Onestep$yearFrom, 1)
  )
  expect_equal(
    tail(demo_cont$lulc_Multistep$yearTo, 1),
    head(demo_cont$lulc_Onestep$yearTo, 1)
  )
  expect_equal(
    head(demo_cont$lulc_Onestep$yearTo, 1) - head(demo_cont$lulc_Multistep$yearFrom, 1),
    demo_cont$totalInterval
  )
  expect_equal(class(demo_cont$tb_legend$categoryName), "factor")
  expect_type(demo_cont$tb_legend$categoryName, "integer")
  expect_equal(as.character(demo_cont$tb_legend$categoryName[3]), "OZS")
  expect_equal(class(demo_cont$tb_legend$color), "character")
  expect_equal(demo_cont$tb_legend$color[4], "#EAACAC")
})

context("input_rasters")

# testFolder <- tempdir()
#
# lapply(.demo_landscape(year = 2000:2004), function(x)
#   raster::writeRaster(x,
#                       filename = file.path(testFolder, paste0(names(x), ".tif")),
#                       datatype = 'INT1U',
#                       overwrite = TRUE
#   ))


test_that("Behavior of the input_raster", {
  # expect_silent(.input_rasters(testFolder))

  expect_error(.input_rasters(c(
    .demo_landscape(year = 2000),
    .demo_landscape(year = 2002, xmx = 50)
  )))

  expect_silent(.input_rasters(.demo_landscape(year = 2000)[[1]]))
  expect_silent(.input_rasters(.demo_landscape(year = 2000)))
  expect_error(.input_rasters(raster::unstack(.input_rasters(.demo_landscape(year = 2000:2004)))))

  expect_error(.input_rasters(list(a = 1, b = 2)))
})


# file.remove(list.files(testFolder, pattern = "landscape", full.names = TRUE))


# test_that("Behavior for case maps not found in the input_raster function", {
#
#   expect_error(.input_rasters(testFolder))
#
#   })

utils::globalVariables(c(
  "Interval", "Period", "Year_from",
  "Year_to", "strings01", "strings02", "changes"
))

#' @include demolandscape.R rasters_input.R
NULL

#' Contingency table
#'
#'
#' Extracts LUC transitions for all input grids of the time series.
#'
#' @param input_raster path (character), Raster* object or list of Raster*
#' objects. See \cr \code{\link[raster]{raster}} for more information about
#' supported file types.
#' @param pixelresolution numeric. The pixel spatial resolution in meter.
#'
#'
#'
#'
#' @import dplyr
#'
#' @return A list that contains 5 objects.
#' \itemize{
#'   \item \code{lulc_Mulstistep}: \code{<tibble>} Contingency table for all
#'   analysed time steps, containing 8 columns:
#'   \enumerate{
#'   \item Period: \code{<chr>} The period \emph{[Yt, Yt+1]}.
#'   \item From: \code{<dbl>} numerical code of a LUC category \emph{i}.
#'   \item To: \code{<dbl>} numerical code of a LUC category \emph{j}.
#'   \item km2: \code{<dbl>} Area in square kilometers that transited from the
#'   category \emph{i}
#'    to category \emph{j} in the period from \emph{Yt} to \emph{Yt+1}.
#'   \item Interval: \code{<dbl>} Interval of years between the first and
#'    the last year of the period \emph{[Yt, Yt+1]}.
#'   \item QtPixel: \code{<int>} Pixel count that transited from the categories
#'    \emph{i}
#'    to category \emph{j} in the period from \emph{Yt} to \emph{Yt+1}.
#'   \item yearFrom: \code{<chr>} The year that the change comes from \emph{[Yt]}.
#'   \item yearTo: \code{<chr>} The year that the change goes for \emph{[Yt+1]}.
#'   }
#'   \item \code{lulc_Onestep}:\code{<tibble>} Contingency table for the entire
#'   analysed period \emph{[Y1, YT]}, containing
#'   8 columns identical with \code{lulc_Mulstistep}.
#'   \item \code{tb_legend}: \code{<tibble>} A table of the pixel value, his
#'   name and color containing 3 columns:
#'   \enumerate{
#'   \item categoryValue: \code{<dbl>} the pixel value of the LUC category.
#'   \item categoryName: \code{<factor>} randomly created string associated with
#'    a given pixel value of a LUC category.
#'   \item color: \code{<chr>} random color associated with the given pixel value
#'    of a LUC category.
#'   Before further analysis, one would like to change the \code{categoryName}
#'   and \code{color} values.
#'     \itemize{
#'         \item Therefore the category names have to be in the same order as the
#'          \code{categoryValue}
#'         and the \code{levels} should be put in the right order for legend
#'         plotting. Like:
#'         \preformatted{
#'
#'         myobject$tb_legend$categoryName <- factor(c("name1", "name2", "name3", "name4"),
#'                                                levels = c("name3", "name2", "name1", "name4"))}
#'         \item The colors have to in the same order as the values in the
#'          \code{categoryValue} column. Colors can be given by the color name
#'          (eg. "black") or an HEX value (eg. #FFFFFF). Like:
#'        \preformatted{
#'
#'        myobject$tb_legend$color <- c("#CDB79E", "red", "#66CD00", "yellow")}}}
#'   \item \code{totalArea}: \code{<tibble>}  A table with the total area of the
#'    study area containing 2 columns:
#'   \enumerate{
#'   \item area_km2: \code{<numeric>} The total area in square kilometers.
#'   \item QtPixel: \code{<numeric>} The total area in pixel counts.
#'   }
#'   \item \code{totalInterval}: \code{<numeric>} Total interval of the analysed
#'    time series in years.
#'   }
#'
#'
#' @export
#'
#' @importFrom raster unstack crosstab compareRaster raster values stack overlay brick
#'
#' @examples
#' \donttest{
#' url <- "https://zenodo.org/record/3685230/files/SaoLourencoBasin.rda?download=1"
#' if (OpenLand:::.openland_try_download_and_load_rda(url,
#'   object = "SaoLourencoBasin", timeout = 10,
#'   cache = FALSE
#' )) {
#'   # the contingencyTable() function, with the SaoLourencoBasin dataset
#'   contingencyTable(input_raster = SaoLourencoBasin, pixelresolution = 30)
#' }
#' }
#'
contingencyTable <-
  function(input_raster, pixelresolution = 30) {
    rList <- .input_rasters(input_raster)

    n_raster <- raster::nlayers(rList)

    if (n_raster < 2) {
      stop("contingencyTable needs at least 2 rasters")
    }

    # compute the cross table of two rasters, then setting the columns name
    table_cross <- function(x, y) {
      contengency <- raster::crosstab(x, y, long = TRUE, progress = "text")
      contengency %>%
        dplyr::mutate(
          Year_from = colnames(contengency)[1],
          Year_to = colnames(contengency)[2]
        ) %>%
        dplyr::rename(
          From = colnames(contengency)[1],
          To = colnames(contengency)[2],
          QtPixel = colnames(contengency)[3]
        ) %>%
        dplyr::mutate(From = as.integer(From), To = as.integer(To))
    }


    table_one <- table_cross(rList[[1]], rList[[raster::nlayers(rList)]])

    if (raster::nlayers(rList) == 2) {
      table_multi <- table_one
    } else {
      rList_multi <- raster::unstack(rList)
      table_multi <- Reduce(
        rbind,
        mapply(
          function(x, y) {
            table_cross(x, y)
          }, rList_multi[1:(length(rList_multi) - 1)],
          rList_multi[2:length(rList_multi)],
          SIMPLIFY = FALSE
        )
      )
    }


    lulc <- list(oneStep = table_one, multiStep = table_multi)

    lulctable <-
      lapply(lulc, function(x) {
        x %>%
          dplyr::arrange(Year_from) %>%
          tidyr::separate(Year_from, c("strings01", "yearFrom"), sep = "_") %>%
          tidyr::separate(Year_to, c("strings02", "yearTo"), sep = "_") %>%
          dplyr::select(-strings01, -strings02) %>%
          dplyr::mutate(
            yearFrom = as.integer(yearFrom), yearTo = as.integer(yearTo),
            Interval = yearTo - yearFrom
          ) %>%
          dplyr::mutate(km2 = QtPixel * (pixelresolution^2) / 1e+06) %>%
          tidyr::unite("Period", c("yearFrom", "yearTo"), sep = "-", remove = FALSE) %>%
          dplyr::select(Period, From, To, km2, QtPixel, Interval, yearFrom, yearTo)
      })


    allinterval <- # calculating the total interval and the pixelValue
      as.numeric(dplyr::last(lulctable[[2]]$yearTo)) - as.numeric(dplyr::first(lulctable[[2]]$yearFrom))

    tb_legend <-
      lulctable[[2]] %>%
      dplyr::distinct(From) %>%
      dplyr::rename(categoryValue = From)

    genCategory <- function() {
      paste(sample(LETTERS, size = 3, replace = FALSE), collapse = "")
    }

    tb_legend$categoryName <- as.factor(vapply(seq_len(nrow(tb_legend)), function(x) genCategory(), character(1)))

    tb_legend$color <- base::sample(
      x = c(
        "#002F70", "#0A468D", "#295EAE", "#4A76C7", "#6F8DD2",
        "#8EA4DE", "#ABBBE8", "#C5CFF0", "#DCE2F6", "#EFF1F8",
        "#F9EFEF", "#F9DCDC", "#F3C5C5", "#EAACAC", "#DD9191",
        "#CE7575", "#BD5758", "#A13F3F", "#7F2A2B", "#5F1415"
      ),
      size = nrow(tb_legend),
      replace = (nrow(tb_legend) > 20)
    )


    areaTotal <-
      lulctable[[2]] %>%
      dplyr::group_by(Period) %>%
      dplyr::summarise(area_km2 = sum(km2), QtPixel = sum(QtPixel))

    contingencyTable <-
      list(
        lulc_Multistep = dplyr::as_tibble(lulctable[[2]]),
        lulc_Onestep = dplyr::as_tibble(lulctable[[1]]),
        tb_legend = dplyr::as_tibble(tb_legend),
        totalArea = areaTotal[1, c(2, 3)],
        totalInterval = allinterval
      )
    contingencyTable
  }

#' @include intensityClass.R
NULL
#' Accessor method for objects from Intensity Analysis
#' @param Interval the class
#' @param x the object
#' @param name the name of the object
#'
#' @keywords internal
#' @rdname acessor
#'
#'


setMethod("$",
  signature = "Interval",
  function(x, name) {
    if (name %in% slotNames(x)) {
      slot(x, name)
    }
  }
)


#' @rdname acessor
#'
#' @param Category the class
setMethod("$",
  signature = "Category",
  function(x, name) {
    if (name %in% slotNames(x)) {
      slot(x, name)
    }
  }
)


#' @rdname acessor
#'
#' @param Transition the class
setMethod("$",
  signature = "Transition",
  function(x, name) {
    if (name %in% slotNames(x)) {
      slot(x, name)
    }
  }
)

#' Download and load a remote .rda dataset (graceful)
#'
#' Internal helper to download a remote `.rda` file (e.g., from Zenodo), load it
#' into an environment, and return a single `TRUE/FALSE` indicating whether the
#' dataset is available.
#'
#' The function is defensive by design: it should not error if the remote host is
#' unavailable, if the download fails, or if the `.rda` cannot be loaded. It can
#' optionally cache the downloaded file in a persistent user cache directory. When
#' `cache = FALSE`, the file is downloaded to a temporary path and removed at the
#' end of the call.
#'
#' @param url character. Remote `.rda` URL.
#' @param object character or NULL. If provided, requires that this object name
#'   exists inside the `.rda` file; otherwise returns `FALSE`.
#' @param envir environment. Environment where the objects from the `.rda` file
#'   will be loaded.
#' @param timeout integer. Download timeout (seconds) set via `options(timeout)`.
#' @param cache logical. Whether to cache the downloaded file.
#' @param cache_dir character or NULL. Cache directory used only when
#'   `cache = TRUE`. When `NULL`, uses `tools::R_user_dir("OpenLand",
#'   which = "cache")` when available.
#' @param quiet logical. Passed to `utils::download.file()`.
#'
#' @return logical. `TRUE` if the `.rda` was successfully downloaded (or found
#'   in cache) and loaded into `envir`; `FALSE` otherwise.
#'
#' @keywords internal
.openland_try_download_and_load_rda <- function(url, # nolint: object_length_linter
                                                object = NULL,
                                                envir = parent.frame(),
                                                timeout = 10,
                                                cache = TRUE,
                                                cache_dir = NULL,
                                                quiet = TRUE) {
  if (!is.character(url) || length(url) != 1L || is.na(url) || !nzchar(url)) {
    return(FALSE)
  }

  timeout <- suppressWarnings(as.integer(timeout))
  if (is.na(timeout) || timeout < 1L) timeout <- 1L

  base_name <- basename(sub("\\?.*$", "", url))
  if (!nzchar(base_name)) base_name <- "OpenLand_zenodo_dataset.rda"
  if (!grepl("\\.[Rr][Dd][Aa]$", base_name)) base_name <- paste0(base_name, ".rda")

  use_cache <- isTRUE(cache)

  if (use_cache) {
    if (is.null(cache_dir)) {
      cache_dir <- if (exists("R_user_dir", envir = asNamespace("tools"), inherits = FALSE)) {
        tools::R_user_dir("OpenLand", which = "cache")
      } else {
        file.path(tempdir(), "OpenLand-cache")
      }
    }

    if (!is.character(cache_dir) || length(cache_dir) != 1L || is.na(cache_dir) || !nzchar(cache_dir)) {
      cache_dir <- tempdir()
    }

    dir.create(cache_dir, recursive = TRUE, showWarnings = FALSE)
    cached_path <- file.path(cache_dir, base_name)
  } else {
    cached_path <- NULL
  }

  tmp_path <- tempfile(pattern = "OpenLand-", fileext = ".rda")
  on.exit(unlink(tmp_path, force = TRUE), add = TRUE)

  old_timeout <- getOption("timeout")
  on.exit(options(timeout = old_timeout), add = TRUE)
  options(timeout = timeout)

  try_load_into <- function(path) {
    tmp_env <- new.env(parent = emptyenv())
    nm <- load(path, envir = tmp_env)
    if (!is.null(object) && !(object %in% nm)) {
      return(FALSE)
    }
    list2env(as.list(tmp_env, all.names = TRUE), envir = envir)
    TRUE
  }

  ok <- tryCatch(
    {
      if (use_cache && file.exists(cached_path) && isTRUE(file.info(cached_path)$size > 0)) {
        if (isTRUE(try_load_into(cached_path))) {
          return(TRUE)
        }
        unlink(cached_path, force = TRUE)
      }

      status <- suppressWarnings(utils::download.file(url, destfile = tmp_path, mode = "wb", quiet = quiet))
      if (!identical(status, 0L)) {
        return(FALSE)
      }

      if (use_cache) {
        ok_copy <- file.copy(tmp_path, cached_path, overwrite = TRUE)
        if (isTRUE(ok_copy)) {
          return(isTRUE(try_load_into(cached_path)))
        }
      }

      isTRUE(try_load_into(tmp_path))
    },
    error = function(e) {
      FALSE
    }
  )

  isTRUE(ok)
}

#' Summary of multiple parameters in a raster directory
#'
#' Listing major characteristics of raster inputs. Those characteristics are the
#' dimensions, the resolution, the extent, the values (min, max) and the
#' coordinate reference system.
#'
#' @param path The path for the Raster* directory or list of Raster* to be
#' analysed.
#'
#' @return Table with the raster parameters in columns
#' @export
#'
#' @examples
#' \donttest{
#' url <- "https://zenodo.org/record/3685230/files/SaoLourencoBasin.rda?download=1"
#' if (OpenLand:::.openland_try_download_and_load_rda(url,
#'   object = "SaoLourencoBasin", timeout = 10,
#'   cache = FALSE
#' )) {
#'   summary_dir(raster::unstack(SaoLourencoBasin))
#' }
#' }
#'
summary_dir <- function(path) {
  if (inherits(path, "list") && inherits(path[[1]], "RasterLayer")) {
    layer_list <- path
  } else if (inherits(path, "character")) {
    raster_files <-
      list.files(path,
        pattern = ".tif$",
        full.names = TRUE
      )

    layer_list <- vector("list", length = length(raster_files))


    for (i in seq_along(raster_files)) {
      layer_list[[i]] <- raster::raster(raster_files[i])
    }
  }

  Reduce(
    rbind,
    lapply(layer_list, function(x) {
      layermap <- x
      tibble(
        file_name = base::names(x),
        xmin = raster::xmin(layermap),
        xmax = raster::xmax(layermap),
        ymin = raster::ymin(layermap),
        ymax = raster::ymax(layermap),
        res_x = raster::res(layermap)[1],
        res_y = raster::res(layermap)[2],
        nrow = raster::nrow(layermap),
        ncol = raster::ncol(layermap),
        min_val = raster::minValue(layermap),
        max_val = raster::maxValue(layermap),
        crs = as.character(raster::crs(layermap))
      )
    })
  )
}


#' Quantitative summary of a unique categorical raster
#'
#' This function presents a summary with the pixel quantity of each category
#' present in a categorical raster.
#'
#' @param path The path for the raster to be analysed, if path is a multilayer
#' raster only the first RasterLayer will be analysed.
#'
#'
#' @return A table containing in columns the pixel counts for each pixel value
#'
#' @export
#'
#' @examples
#' \donttest{
#' url <- "https://zenodo.org/record/3685230/files/SaoLourencoBasin.rda?download=1"
#' if (OpenLand:::.openland_try_download_and_load_rda(url,
#'   object = "SaoLourencoBasin", timeout = 10,
#'   cache = FALSE
#' )) {
#'   summary_map(SaoLourencoBasin[[1]])
#' }
#' }
#'
summary_map <- function(path) {
  rastermap <-
    if (!inherits(path, "character")) {
      if (inherits(path, "RasterLayer")) {
        path
      } else {
        path[[1]]
      }
    } else {
      raster::raster(path)
    }
  value_map <- table(raster::values(rastermap))

  tbfinal <- dplyr::tibble(
    pixvalue = numeric(length(value_map)),
    Qt = numeric(length(value_map))
  )

  for (i in seq_along(value_map)) {
    tbfinal[i, c(1:2)] <-
      list(as.numeric(names(value_map)[i]), value_map[[i]])
  }
  tbfinal
}


#' Accumulates changes in a LULC raster time series
#'
#' This function calculates the number of times a pixel has changed during
#' the analysed period. It returns a raster with the number of changes as
#' pixel value and a table containing the areal percentage of every pixel value
#' (number of changes).
#'
#'
#' @param path The path for the Raster* directory or list of Raster* to be
#' analysed.
#'
#'
#' @return Two objects, a RasterLayer and a table.
#' @export
#'
#'
#'
#' @examples
#' \donttest{
#' url <- "https://zenodo.org/record/3685230/files/SaoLourencoBasin.rda?download=1"
#' if (OpenLand:::.openland_try_download_and_load_rda(url,
#'   object = "SaoLourencoBasin", timeout = 10,
#'   cache = FALSE
#' )) {
#'   acc_changes(SaoLourencoBasin)
#' }
#' }
#'
acc_changes <- function(path) {
  rList <- .input_rasters(path)

  rList <- raster::unstack(rList)

  n_raster <- length(rList)

  if (n_raster < 2) {
    stop("acc_changes needs at least 2 rasters")
  }

  difflist <- mapply(
    function(x, y) {
      raster::overlay(
        x,
        y,
        fun = function(x1, x2) {
          ifelse((x1 != x2), 1, 0)
        }
      )
    },
    x = rList[1:(length(rList) - 1)],
    y = rList[2:length(rList)],
    SIMPLIFY = FALSE
  )

  sumraster <- sum(raster::stack(difflist))

  Freq <- Var1 <- NULL

  df01_values <- table(matrix(sumraster))

  df_values <- dplyr::mutate(data.frame(df01_values),
    Var1 = as.character(Var1),
    Var1 = as.integer(Var1),
    Percent = Freq / sum(Freq) * 100
  )

  df_values <- dplyr::as_tibble(df_values)

  names(df_values) <- c("PxValue", "Qt", "Percent")

  list(sumraster, df_values)
}

context("generalfunctions")

set.seed(159)
demo_raster <- .demo_landscape(
  year = 2000:2005,
  res = 1,
  crs = "+proj=utm +zone=21 +south +ellps=GRS80 +units=m +no_defs"
)
demo_acc_changes <- acc_changes(demo_raster)

test_that("Behavior of acc_changes", {
  expect_error(acc_changes(.demo_landscape(2000)))
  expect_error(acc_changes(2000))
  expect_silent(acc_changes(demo_raster))
  expect_silent(acc_changes(raster::stack(demo_raster)))
  expect_silent(acc_changes(raster::brick(demo_raster)))
  expect_length(demo_acc_changes, 2)
  expect_output(str(demo_acc_changes), "List of 2")
  expect_match(class(demo_acc_changes[[1]]), "RasterLayer")
  expect_setequal(class(demo_acc_changes[[2]]), c("tbl_df", "tbl", "data.frame"))
  expect_equal(sum(demo_acc_changes[[2]][, 3]), 100)
  expect_equal(demo_acc_changes[[2]][[4, 2]], 1998)
  expect_equal(ncol(demo_acc_changes[[2]]), 3)
})


if (FALSE) {
  testFolder <- tempdir()

  lapply(.demo_landscape(year = 2000:2004), function(x) {
    raster::writeRaster(x,
      filename = file.path(testFolder, paste0(names(x), ".tif")),
      datatype = "INT1U",
      overwrite = TRUE
    )
  })

  expect_silent(summary_dir(testFolder))
  expect_equal(nrow(summary_dir(testFolder)), 5)
}


test_that("Behavior of summary_dir", {
  expect_silent(summary_dir(demo_raster))
  expect_visible(summary_dir(demo_raster))
  expect_error(summary_dir(raster::stack(demo_raster)))
  expect_error(summary_dir(raster::brick(demo_raster)))
  expect_equal(ncol(summary_dir(demo_raster)), 12)
  expect_equal(nrow(summary_dir(demo_raster)), length(demo_raster))
  expect_setequal(class(summary_dir(demo_raster)), c("tbl_df", "tbl", "data.frame"))
})


test_that("summary_dir and summary_map work with file paths", {
  testFolder <- tempfile(pattern = "OpenLand-raster-dir-")
  dir.create(testFolder)

  demo_small <- .demo_landscape(
    year = 2000:2002,
    res = 1,
    crs = NA
  )

  lapply(demo_small, function(x) {
    raster::writeRaster(x,
      filename = file.path(testFolder, paste0(names(x), ".tif")),
      datatype = "INT1U",
      overwrite = TRUE
    )
  })

  out_dir <- summary_dir(testFolder)
  expect_setequal(class(out_dir), c("tbl_df", "tbl", "data.frame"))
  expect_equal(nrow(out_dir), length(demo_small))

  tif1 <- list.files(testFolder, pattern = "\\.tif$", full.names = TRUE)[1]
  out_map <- summary_map(tif1)
  expect_setequal(class(out_map), c("tbl_df", "tbl", "data.frame"))
  expect_equal(ncol(out_map), 2)
})


test_that("Behavior of summary_map", {
  # expect_silent(summary_map(list.files(testFolder, pattern = "tif$", full.names = TRUE)[1]))
  # expect_silent(summary_map(.input_rasters(testFolder)))
  expect_silent(summary_map(demo_raster[[1]]))
  expect_visible(summary_map(demo_raster[[1]]))
  expect_equal(ncol(summary_map(demo_raster[[1]])), 2)
  expect_equal(nrow(summary_map(demo_raster[[1]])), 5)
})


test_that(".openland_try_download_and_load_rda loads from cache", {
  cache_dir <- tempfile(pattern = "OpenLand-cache-")
  dir.create(cache_dir)

  url <- "https://example.com/SaoLourencoBasin.rda?download=1"
  cached_path <- file.path(cache_dir, "SaoLourencoBasin.rda")

  a <- 123
  save(a, file = cached_path)

  env <- new.env(parent = emptyenv())
  ok <- OpenLand:::.openland_try_download_and_load_rda(url,
    object = "a",
    envir = env,
    cache = TRUE,
    cache_dir = cache_dir,
    timeout = 1,
    quiet = TRUE
  )

  expect_true(ok)
  expect_true(exists("a", envir = env, inherits = FALSE))
  expect_identical(env$a, 123)
})


test_that(".openland_try_download_and_load_rda handles missing .rda extension", {
  cache_dir <- tempfile(pattern = "OpenLand-cache-")
  dir.create(cache_dir)

  url <- "https://example.com/foo"
  cached_path <- file.path(cache_dir, "foo.rda")

  b <- 999
  save(b, file = cached_path)

  env <- new.env(parent = emptyenv())
  ok <- OpenLand:::.openland_try_download_and_load_rda(url,
    object = "b",
    envir = env,
    cache = TRUE,
    cache_dir = cache_dir,
    timeout = 1,
    quiet = TRUE
  )

  expect_true(ok)
  expect_true(exists("b", envir = env, inherits = FALSE))
  expect_identical(env$b, 999)
})


test_that(".openland_try_download_and_load_rda invalidates bad cache then downloads from file URL", {
  as_file_url <- function(path) {
    p <- normalizePath(path, winslash = "/", mustWork = TRUE)
    if (grepl("^[A-Za-z]:", p)) paste0("file:///", p) else paste0("file://", p)
  }

  src <- tempfile(pattern = "OpenLand-src-", fileext = ".rda")
  b <- 42
  save(b, file = src)
  url <- as_file_url(src)

  cache_dir <- tempfile(pattern = "OpenLand-cache-")
  dir.create(cache_dir)

  cached_path <- file.path(cache_dir, basename(sub("\\?.*$", "", url)))
  a <- 7
  save(a, file = cached_path)

  env <- new.env(parent = emptyenv())
  ok <- OpenLand:::.openland_try_download_and_load_rda(url,
    object = "b",
    envir = env,
    cache = TRUE,
    cache_dir = cache_dir,
    timeout = 5,
    quiet = TRUE
  )

  expect_true(ok)
  expect_true(exists("b", envir = env, inherits = FALSE))
  expect_identical(env$b, 42)

  env_cached <- new.env(parent = emptyenv())
  load(cached_path, envir = env_cached)
  expect_true(exists("b", envir = env_cached, inherits = FALSE))
})


test_that(".openland_try_download_and_load_rda cache=FALSE still loads from file URL", {
  as_file_url <- function(path) {
    p <- normalizePath(path, winslash = "/", mustWork = TRUE)
    if (grepl("^[A-Za-z]:", p)) paste0("file:///", p) else paste0("file://", p)
  }

  src <- tempfile(pattern = "OpenLand-src-", fileext = ".rda")
  c_obj <- "ok"
  save(c_obj, file = src)
  url <- as_file_url(src)

  cache_dir <- tempfile(pattern = "OpenLand-cache-")

  env <- new.env(parent = emptyenv())
  ok <- OpenLand:::.openland_try_download_and_load_rda(url,
    object = "c_obj",
    envir = env,
    cache = FALSE,
    cache_dir = cache_dir,
    timeout = 5,
    quiet = TRUE
  )

  expect_true(ok)
  expect_true(exists("c_obj", envir = env, inherits = FALSE))
  expect_identical(env$c_obj, "ok")

  cached_path <- file.path(cache_dir, basename(sub("\\?.*$", "", url)))
  expect_false(dir.exists(cache_dir))
  expect_false(file.exists(cached_path))
})


test_that(".openland_try_download_and_load_rda returns FALSE for invalid url", {
  env <- new.env(parent = emptyenv())
  expect_false(OpenLand:::.openland_try_download_and_load_rda("", envir = env))
  expect_false(OpenLand:::.openland_try_download_and_load_rda(NA_character_, envir = env))
})

# file.remove(list.files(testFolder, pattern = "landscape", full.names = TRUE))

## ----setup, include = FALSE---------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)

## ----include=FALSE------------------------------------------------------------
url <- "https://zenodo.org/record/3685230/files/SaoLourencoBasin.rda?download=1"
dataset_available <- OpenLand:::.openland_try_download_and_load_rda(url,
  object = "SaoLourencoBasin",
  timeout = 10,
  cache = FALSE,
  quiet = TRUE
)

## ----include=FALSE, eval=isFALSE(dataset_available)---------------------------
knitr::knit_exit(paste0(
  "## The complete rendering of this document depends on the availability of the dataset in zenodo. ",
  "Please try at another time."
))

#' input_rasters
#'
#' A methods for loading the raster into OpenLand
#'
#' @param x path (character), Raster* object or list of Raster* objects.
#' @param \dots additional arguments to \code{raster::\link[raster]{raster}}.
#'
#' @return A RasterStack
#'
#' @seealso \code{raster::\link[raster]{raster}},
#' \code{raster::\link[raster]{stack}}
#'
#' @keywords internal
#' @rdname dot-input_rasters
#' @exportMethod .input_rasters
#'


setGeneric(".input_rasters", function(x, ...) {
  standardGeneric(".input_rasters")
})


# the methods character
#' @rdname dot-input_rasters
#' @aliases character
setMethod(
  ".input_rasters", signature(x = "character"),
  function(x, ...) {
    files <- list.files(
      path = x,
      pattern = ".tif$",
      full.names = FALSE
    )
    if (length(files) > 0) {
      sorted_files <- sort(files)
      paths <- file.path(x, sorted_files)
      maps <- raster::stack(paths)
      maps
    } else {
      stop("maps not found")
    }
  }
)

#' @rdname dot-input_rasters
#' @aliases list
setMethod(
  ".input_rasters", signature(x = "list"),
  function(x, ...) {
    list.names <- names(x)
    if (is.null(list.names)) {
      stop("list elements must be named")
    }
    files <- list.names
    sorted_files <- sort(files)
    maps <- raster::stack(x[sorted_files])
    maps
  }
)

#' @rdname dot-input_rasters
#' @aliases RasterLayer
setMethod(
  ".input_rasters", signature(x = "RasterLayer"),
  function(x, ...) {
    maps <- raster::stack(x)
    maps
  }
)

#' @rdname dot-input_rasters
#' @aliases RasterBrick
setMethod(
  ".input_rasters", signature(x = "RasterBrick"),
  function(x, ...) {
    maps <- x
    maps
  }
)

#' @rdname dot-input_rasters
#' @aliases RasterStack
setMethod(
  ".input_rasters", signature(x = "RasterStack"),
  function(x, ...) {
    maps <- x
    maps
  }
)

"_PACKAGE"
#' OpenLand: land use and cover (LUC) time series analysis in R.
#'
#'
#' OpenLand is an open-source R package for the analysis of land use and cover
#' (LUC) time series. It includes support for consistency check and loading
#' spatiotemporal raster data and synthesized spatial plotting. Several LUC change
#' (LUCC) metrics in regular or irregular time intervals can be extracted and
#' visualized through one- and multistep sankey and chord diagrams. A complete
#' intensity analysis according to \cite{(Aldwaik and Pontius, 2012)} is
#' implemented, including tools for the generation of standardized multilevel
#' output graphics.
#'
#'
#' @seealso The core functions in this package: \code{\link{intensityAnalysis}},
#' \code{\link{contingencyTable}},
#'
#' @author Reginal Exavier \email{reginalexavier@@rocketmail.com}, Peter Zeilhofer \email{zeilhoferpeter@@gmail.com}
# @docType package
#' @name OpenLand-package
#' @aliases OpenLand
#'
#'
#'
#' @references
#' Aldwaik, S. Z. and Pontius, R. G. (2012) ‘Intensity analysis to unify
#' measurements of size and stationarity of land changes by interval, category,
#' and transition, Landscape and Urban Planning. Elsevier B.V., 106(1), pp. 103–114.
#' \doi{10.1016/j.landurbplan.2012.02.010}.
#'
#'
#'
#'
#'
NULL

utils::globalVariables(c(
  "Gain", "Gtj", "Loss", "Lti", "N", "Qtmj",
  "Rtin", "St", "Vtm", "Wtn", "intch_QtPixel",
  "intch_km2", "num02"
))
#' @include intensityClass.R
NULL


#' Performs the intensity analysis based on cross-tabulation matrices of each
#' time step
#'
#' This function implements an Intensity Analysis (IA) according to Aldwaik &
#' Pontius (2012), a quantitative method to analyze time series of land use and
#' cover (LUC) maps.  For IA, a cross-tabulation matrix is composed for each LUC
#' transition step in time.
#'
#' IA includes three levels of analysis of LUC changes. Consecutive analysis
#' levels detail hereby information given by the previous analysis level
#' \cite{(Aldwaik and Pontius, 2012, 2013)}.
#'
#'
#' \enumerate{
#'  \item The \emph{interval level} examines how the size and speed of change
#'  vary across time intervals.
#'  \item The \emph{category level} examines how the size and intensity of gross
#'  losses and gross gains in each category vary across categories for each time
#'  interval.
#'  \item The \emph{transition level} examines how the size and intensity of a
#'  category’s transitions vary across the other categories that are available
#'  for that transition.
#'   }
#'
#'
#' At each analysis level, the method tests for stationarity of patterns across
#' time intervals.
#'
#' \bold{The function returns a list with 6 objects:}
#' \enumerate{
#'  \item lulc_table: \code{tibble}. Contingency table of LUC transitions at all
#'   analysed time steps, containing 6 columns:
#'    \enumerate{
#'    \item Period:  \code{<fct>}. Evaluated period of transition in the format
#'    \code{year t - year t+1}.
#'    \item From: \code{<fct>}. The category in year t.
#'    \item To: \code{<fct>}. The category in year t+1.
#'    \item km2: \code{<dbl>}. Area in square kilometers that transited from the
#'     category \code{From}.
#'    to the category \code{To} in the period.
#'    \item QtPixel: \code{<int>}. Number of pixels that transited from.
#'    the category \code{From} to the category \code{To} in the period.
#'    \item Interval: \code{<int>}. Interval in years of the evaluated period.
#'
#'    }
#'
#'  \item \emph{lv1_tbl}: An \code{\linkS4class{Interval}} object containing the
#'   \emph{St} and \emph{U} values.
#'  \item \emph{category_lvlGain}: A \code{\linkS4class{Category}} object
#'  containing the gain of the LUC category in a period (\emph{Gtj}).
#'  \item \emph{category_lvlLoss}: A \code{\linkS4class{Category}} object
#'  containing the loss of the LUC category in a period (\emph{Lti}).
#'  \item \emph{transition_lvlGain_n}: A \code{\linkS4class{Transition}} object
#'  containing the annualized rate of gain in \emph{category n} (\emph{Rtin}) and
#'  the respective Uniform Intensity (\emph{Wtn}).
#'  \item \emph{transition_lvlLoss_m}: A \code{\linkS4class{Transition}} object
#'  containing the annualized rate of loss in \emph{category m} (\emph{Qtmj}) and
#'  the respective Uniform Intensity (\emph{Vtm}).
#'
#'   }
#'
#'
#'
#'
#' @param dataset list. The result object from \code{\link{contingencyTable}}.
#' @param category_n character. The gaining category in the transition of interest (\emph{n}).
#' @param category_m character. The losing category in the transition of interest (\emph{m}).
#' @param area_km2 logical. If TRUE the change is computed in km2, if FALSE in pixel counts.
#'
#' @return Intensity object
#' @export
#'
#'
#' @references Aldwaik, S. Z. and Pontius, R. G. (2012) ‘Intensity analysis to unify
#' measurements of size and stationarity of land changes by interval, category, and
#' transition, Landscape and Urban Planning. Elsevier B.V., 106(1), pp. 103–114.
#' \doi{10.1016/j.landurbplan.2012.02.010}.
#'
#' Aldwaik, S. Z. and Pontius, R. G. (2013) ‘Map errors that could account for deviations
#' from a uniform intensity of land change, International Journal of Geographical
#' Information Science. Taylor & Francis, 27(9), pp. 1717–1739. \doi{10.1080/13658816.2013.787618}.
#'
#'
#'
#' @examples
#'
#' # editing the category name
#'
#' SL_2002_2014$tb_legend$categoryName <- factor(
#'   c(
#'     "Ap", "FF", "SA", "SG", "aa", "SF",
#'     "Agua", "Iu", "Ac", "R", "Im"
#'   ),
#'   levels = c(
#'     "FF", "SF", "SA", "SG", "aa", "Ap",
#'     "Ac", "Im", "Iu", "Agua", "R"
#'   )
#' )
#'
#  # add the color by the same order of the legend factor
#' SL_2002_2014$tb_legend$color <- c(
#'   "#FFE4B5", "#228B22", "#00FF00", "#CAFF70",
#'   "#EE6363", "#00CD00", "#436EEE", "#FFAEB9",
#'   "#FFA54F", "#68228B", "#636363"
#' )
#'
#' intensityAnalysis(dataset = SL_2002_2014, category_n = "Ap", category_m = "SG", area_km2 = TRUE)
#'
intensityAnalysis <-
  function(dataset, category_n, category_m, area_km2 = TRUE) {
    # setting the data
    AE <- dataset[[4]] # study area

    allinterval <-
      dataset[[5]] # whole interval in years

    lulc <-
      dplyr::left_join(dataset[[1]], dataset[[3]][, c(1, 2)], by = c("From" = "categoryValue")) %>%
      dplyr::left_join(dataset[[3]][, c(1, 2)], by = c("To" = "categoryValue")) %>%
      dplyr::select(-c(From, To)) %>%
      dplyr::rename("From" = "categoryName.x", "To" = "categoryName.y") %>%
      dplyr::select(Period, From, To, km2, QtPixel, Interval)


    lulc$Period <-
      factor(as.factor(lulc$Period), levels = rev(levels(as.factor(lulc$Period))))

    category_fillColor <- dataset[[3]][c(2, 3)]

    lookupcolor <- category_fillColor$color

    names(lookupcolor) <- category_fillColor$categoryName


    if (isTRUE(area_km2)) {
      # ____________Interval-------
      # EQ1 - St ----

      eq1 <- lulc %>%
        dplyr::filter(From != To) %>%
        dplyr::group_by(Period, Interval) %>%
        dplyr::summarise(intch_km2 = sum(km2)) %>% # interval change:intch_km2
        dplyr::mutate(
          PercentChange = (intch_km2 / AE[[1, 1]]) * 100,
          St = (intch_km2 / (Interval * AE[[1, 1]])) * 100
        ) %>%
        dplyr::select(1, 4, 5)

      # EQ2 - U ----
      eq2 <- lulc %>%
        dplyr::filter(From != To) %>%
        dplyr::summarise(num02 = sum(km2)) %>%
        dplyr::mutate(U = (num02 / (allinterval * AE[[1, 1]])) * 100)

      level01 <-
        eq1 %>% dplyr::mutate(U = eq2[[2]]) # Type = ifelse(St > U, "Fast", "Slow"))

      # ____________Categorical ----
      # EQ3 - Gtj ----
      num03 <- lulc %>%
        dplyr::filter(From != To) %>%
        dplyr::group_by(Period, To, Interval) %>%
        dplyr::summarise(num03 = sum(km2))

      denom03 <-
        lulc %>%
        dplyr::group_by(Period, To) %>%
        dplyr::summarise(denom03 = sum(km2))

      eq3 <-
        num03 %>%
        dplyr::left_join(denom03, by = c("Period", "To")) %>%
        dplyr::mutate(Gtj = (num03 / (denom03 * Interval)) * 100) %>%
        dplyr::left_join(eq1[c(1, 3)], by = "Period") %>%
        dplyr::select(1, 2, 3, 4, 6, 7) %>%
        rename("GG_km2" = "num03")

      # EQ4 -   Lti ---------
      num04 <- lulc %>%
        dplyr::filter(From != To) %>%
        dplyr::group_by(Period, From, Interval) %>%
        dplyr::summarise(num04 = sum(km2))

      denom04 <-
        lulc %>%
        dplyr::group_by(Period, From) %>%
        dplyr::summarise(denom04 = sum(km2))

      eq4 <-
        num04 %>%
        dplyr::left_join(denom04, by = c("Period", "From")) %>%
        dplyr::mutate(Lti = (num04 / (denom04 * Interval)) * 100) %>%
        dplyr::left_join(eq1[c(1, 3)], by = "Period") %>%
        dplyr::select(1, 2, 3, 4, 6, 7) %>%
        rename("GL_km2" = "num04")

      # ____________Transition ----
      # Witch transitions are particularly intensive in a given time interval?
      # EQ5 - Rtin----
      num05 <-
        lulc %>%
        dplyr::filter(From != To, To == category_n) %>%
        dplyr::group_by(Period, From, Interval) %>%
        dplyr::summarise(num05 = sum(km2))

      denom05 <-
        lulc %>%
        dplyr::filter(From != category_n) %>%
        dplyr::group_by(Period, From) %>%
        dplyr::summarise(denom05 = sum(km2))

      eq5 <-
        num05 %>%
        dplyr::left_join(denom05, by = c("Period", "From")) %>%
        dplyr::mutate(Rtin = (num05 / (Interval * denom05)) * 100)

      # EQ6 - Wtn ----
      num06 <- lulc %>%
        dplyr::filter(From != To, To == category_n) %>%
        dplyr::group_by(Period, To, Interval) %>%
        dplyr::summarise(num06 = sum(km2))

      denom06 <- lulc %>%
        dplyr::filter(From != category_n) %>%
        dplyr::group_by(Period) %>%
        dplyr::summarise(denom06 = sum(km2))

      eq6 <- num06 %>%
        dplyr::left_join(denom06, by = "Period") %>%
        dplyr::mutate(Wtn = (num06 / (Interval * denom06)) * 100)

      plot03ganho_n <-
        eq5 %>%
        dplyr::left_join(eq6, by = c("Period", "Interval")) %>%
        dplyr::select(1, 2, 7, 3, 4, 6, 10) %>%
        rename("T_i2n_km2" = "num05")

      # EQ7 - Qtmj----
      num07 <- lulc %>%
        dplyr::filter(From != To, From == category_m) %>%
        dplyr::group_by(Period, To, Interval) %>%
        dplyr::summarise(num07 = sum(km2))

      denom07 <- lulc %>%
        dplyr::filter(To != category_m) %>%
        dplyr::group_by(Period, To) %>%
        dplyr::summarise(denom07 = sum(km2))

      eq7 <-
        num07 %>%
        dplyr::left_join(denom07, by = c("Period", "To")) %>%
        dplyr::mutate(Qtmj = (num07 / (Interval * denom07)) * 100)

      # EQ8 - Vtm----
      num08 <- lulc %>%
        dplyr::filter(From != To, From == category_m) %>%
        dplyr::group_by(Period, From, Interval) %>%
        dplyr::summarise(num08 = sum(km2))

      denom08 <- lulc %>%
        dplyr::filter(To != category_m) %>%
        dplyr::group_by(Period) %>%
        dplyr::summarise(denom08 = sum(km2))


      eq8 <- num08 %>%
        dplyr::left_join(denom08, by = "Period") %>%
        dplyr::mutate(Vtm = (num08 / (Interval * denom08)) * 100)

      plot03perda_m <-
        eq7 %>%
        dplyr::left_join(eq8, by = c("Period", "Interval")) %>%
        dplyr::select(1, 2, 7, 3, 4, 6, 10) %>%
        rename("T_m2j_km2" = "num07")
    } else {
      # ____________Interval-------
      # EQ1 - St ----

      eq1 <- lulc %>%
        dplyr::filter(From != To) %>%
        dplyr::group_by(Period, Interval) %>%
        dplyr::summarise(intch_QtPixel = sum(QtPixel)) %>% # interval change:intch_QtPixel
        dplyr::mutate(
          PercentChange = (intch_QtPixel / AE[[1, 2]]) * 100,
          St = (intch_QtPixel / (Interval * AE[[1, 2]])) * 100
        ) %>%
        dplyr::select(1, 4, 5)

      # EQ2 - U ----
      eq2 <- lulc %>%
        dplyr::filter(From != To) %>%
        dplyr::summarise(num02 = sum(QtPixel)) %>% # all area change for the whole period Y1 to YT
        dplyr::mutate(U = (num02 / (allinterval * AE[[1, 2]])) * 100)

      level01 <-
        eq1 %>% dplyr::mutate(U = eq2[[2]]) # Type = ifelse(St > U, "Fast", "Slow"))

      # ____________Categorical ----
      # EQ3 - Gtj ----
      num03 <- lulc %>%
        dplyr::filter(From != To) %>%
        dplyr::group_by(Period, To, Interval) %>%
        dplyr::summarise(num03 = sum(QtPixel)) # gross gain category in time point Yt+1

      denom03 <-
        lulc %>%
        dplyr::group_by(Period, To) %>%
        dplyr::summarise(denom03 = sum(QtPixel)) # total area category in time point Yt+1

      eq3 <-
        num03 %>%
        dplyr::left_join(denom03, by = c("Period", "To")) %>%
        dplyr::mutate(Gtj = (num03 / (denom03 * Interval)) * 100) %>%
        dplyr::left_join(eq1[c(1, 3)], by = "Period") %>%
        dplyr::select(1, 2, 3, 4, 6, 7) %>%
        rename("GG_pixel" = "num03")

      # EQ4 -   Lti ---------
      num04 <- lulc %>%
        dplyr::filter(From != To) %>%
        dplyr::group_by(Period, From, Interval) %>%
        dplyr::summarise(num04 = sum(QtPixel)) # gross loss of category i in time point Yt

      denom04 <-
        lulc %>%
        dplyr::group_by(Period, From) %>%
        dplyr::summarise(denom04 = sum(QtPixel)) # total area of the category in time point Yt

      eq4 <-
        num04 %>%
        dplyr::left_join(denom04, by = c("Period", "From")) %>%
        dplyr::mutate(Lti = (num04 / (denom04 * Interval)) * 100) %>%
        dplyr::left_join(eq1[c(1, 3)], by = "Period") %>%
        dplyr::select(1, 2, 3, 4, 6, 7) %>%
        rename("GL_pixel" = "num04")

      # ____________Transition ----
      # Witch transitions are particularly intensive in a given time interval?
      # EQ5 - Rtin----
      num05 <-
        lulc %>%
        dplyr::filter(From != To, To == category_n) %>%
        dplyr::group_by(Period, From, Interval) %>%
        dplyr::summarise(num05 = sum(QtPixel)) # transition area for every i to n

      denom05 <-
        lulc %>%
        dplyr::filter(From != category_n) %>% # filtering all non-n
        dplyr::group_by(Period, From) %>%
        dplyr::summarise(denom05 = sum(QtPixel)) # total area of every i in the time point Yt

      eq5 <-
        num05 %>%
        dplyr::left_join(denom05, by = c("Period", "From")) %>%
        dplyr::mutate(Rtin = (num05 / (Interval * denom05)) * 100)

      # EQ6 - Wtn ----
      num06 <- lulc %>%
        dplyr::filter(From != To, To == category_n) %>%
        dplyr::group_by(Period, To, Interval) %>%
        dplyr::summarise(num06 = sum(QtPixel)) # gross gain of n during the transition

      denom06 <- lulc %>%
        dplyr::filter(From != category_n) %>%
        dplyr::group_by(Period) %>%
        dplyr::summarise(denom06 = sum(QtPixel)) # non-n area in time point Yt

      eq6 <- num06 %>%
        dplyr::left_join(denom06, by = "Period") %>%
        dplyr::mutate(Wtn = (num06 / (Interval * denom06)) * 100)

      plot03ganho_n <-
        eq5 %>%
        dplyr::left_join(eq6, by = c("Period", "Interval")) %>%
        dplyr::select(1, 2, 7, 3, 4, 6, 10) %>%
        rename("T_i2n_pixel" = "num05")


      # EQ7 - Qtmj----
      num07 <- lulc %>%
        dplyr::filter(From != To, From == category_m) %>%
        dplyr::group_by(Period, To, Interval) %>%
        dplyr::summarise(num07 = sum(QtPixel)) # trasition area from m to every j

      denom07 <- lulc %>%
        dplyr::filter(To != category_m) %>%
        dplyr::group_by(Period, To) %>%
        dplyr::summarise(denom07 = sum(QtPixel)) # total area of every j in time point (Yt+1)

      eq7 <-
        num07 %>%
        dplyr::left_join(denom07, by = c("Period", "To")) %>%
        dplyr::mutate(Qtmj = (num07 / (Interval * denom07)) * 100)

      # EQ8 - Vtm----
      num08 <- lulc %>%
        dplyr::filter(From != To, From == category_m) %>%
        dplyr::group_by(Period, From, Interval) %>%
        dplyr::summarise(num08 = sum(QtPixel)) # gross loss of m during the transition

      denom08 <- lulc %>%
        dplyr::filter(To != category_m) %>%
        dplyr::group_by(Period) %>%
        dplyr::summarise(denom08 = sum(QtPixel)) # non-m area in the time point Y(t+1)


      eq8 <- num08 %>%
        dplyr::left_join(denom08, by = "Period") %>%
        dplyr::mutate(Vtm = (num08 / (Interval * denom08)) * 100)

      plot03perda_m <-
        eq7 %>%
        dplyr::left_join(eq8, by = c("Period", "Interval")) %>%
        dplyr::select(1, 2, 7, 3, 4, 6, 10) %>%
        rename("T_m2j_pixel" = "num07")
    }

    # ___Stationarity test------

    # Level02 ----
    # gain
    st_lv2_gain <-
      eq3 %>%
      dplyr::filter(Gtj > St) %>%
      dplyr::group_by(To) %>%
      dplyr::summarise(
        Gain = dplyr::n(),
        N = length(unique(eq3$Period)),
        Stationarity = "Active Gain",
        Test = ifelse(Gain == N, "Y", "N")
      ) %>%
      rbind(
        eq3 %>%
          dplyr::filter(Gtj < St) %>%
          dplyr::group_by(To) %>%
          dplyr::summarise(
            Gain = dplyr::n(),
            N = length(unique(eq3$Period)),
            Stationarity = "Dormant Gain",
            Test = ifelse(Gain == N, "Y", "N")
          )
      )

    # loss
    st_lv2_loss <-
      eq4 %>%
      dplyr::filter(Lti > St) %>%
      dplyr::group_by(From) %>%
      dplyr::summarise(
        Loss = dplyr::n(),
        N = length(unique(eq4$Period)),
        Stationarity = "Active Loss",
        Test = ifelse(Loss == N, "Y", "N")
      ) %>%
      rbind(
        eq4 %>%
          dplyr::filter(Lti < St) %>%
          dplyr::group_by(From) %>%
          dplyr::summarise(
            Loss = dplyr::n(),
            N = length(unique(eq4$Period)),
            Stationarity = "Dormant Loss",
            Test = ifelse(Loss == N, "Y", "N")
          )
      )

    # Level03 gain_n ----
    st_gain_n <-
      plot03ganho_n %>%
      dplyr::filter(Rtin > Wtn) %>%
      dplyr::group_by(From) %>%
      dplyr::summarise(
        Loss = dplyr::n(),
        N = length(unique(plot03ganho_n$Period)),
        Stationarity = paste("targeted by", category_n),
        Test = ifelse(Loss == N, "Y", "N")
      ) %>%
      rbind(
        plot03ganho_n %>%
          dplyr::filter(Rtin < Wtn) %>%
          dplyr::group_by(From) %>%
          dplyr::summarise(
            Loss = dplyr::n(),
            N = length(unique(plot03ganho_n$Period)),
            Stationarity = paste("avoided by", category_n),
            Test = ifelse(Loss == N, "Y", "N")
          )
      )

    # Level03b loss_m----
    st_loss_m <-
      plot03perda_m %>%
      dplyr::filter(Qtmj > Vtm) %>%
      dplyr::group_by(To) %>%
      dplyr::summarise(
        Gain = dplyr::n(),
        N = length(unique(plot03perda_m$Period)),
        Stationarity = paste("targeted", category_m),
        Test = ifelse(Gain == N, "Y", "N")
      ) %>%
      rbind(
        plot03perda_m %>%
          dplyr::filter(Qtmj < Vtm) %>%
          dplyr::group_by(To) %>%
          dplyr::summarise(
            Gain = dplyr::n(),
            N = length(unique(plot03perda_m$Period)),
            Stationarity = paste("avoided", category_m),
            Test = ifelse(Gain == N, "Y", "N")
          )
      )

    # Instances of intensity class ----
    intensity_tables <-
      list(
        lulc_table = lulc,
        interval_lvl = new("Interval",
          intervalData = level01
        ),
        category_lvlGain = new(
          "Category",
          categoryData = eq3,
          lookupcolor = lookupcolor,
          categoryStationarity = st_lv2_gain
        ),
        category_lvlLoss = new(
          "Category",
          categoryData = eq4,
          lookupcolor = lookupcolor,
          categoryStationarity = st_lv2_loss
        ),
        transition_lvlGain_n = new(
          "Transition",
          transitionData = plot03ganho_n,
          lookupcolor = lookupcolor,
          transitionStationarity = st_gain_n
        ),
        transition_lvlLoss_m = new(
          "Transition",
          transitionData = plot03perda_m,
          lookupcolor = lookupcolor,
          transitionStationarity = st_loss_m
        )
      )
    intensity_tables
  }

#' Tables of land use and cover (LUC) in the São Lourenço River Basin (2002 - 2014)
#'
#' A list containing five objects created by the \code{\link{contingencyTable}}
#' function with \code{SaoLourencoBasin} as input
#' (\code{SL_2002_2014 <- contingenceTable(input_raster = SaoLourencoBasin, pixelresolution = 30)}).
#'
#'
#' @docType data
#'
#' @usage data(SL_2002_2014)
#'
#' @format A data list with 5 objects:
#' \describe{
#'   \item{lulc_Multistep}{\code{<tibble>} Contingency table for all analysed
#'   time steps, containing 8 columns:
#'   \enumerate{
#'   \item Period: \code{<chr>} The period \emph{[Yt, Yt+1]}.
#'   \item From: \code{<int>} numerical code of a LUC category \emph{i}.
#'   \item To: \code{<int>} numerical code of a LUC category \emph{j}.
#'   \item km2: \code{<dbl>} Area in square kilometers that transited from the
#'   category \emph{i}
#'    to category \emph{j} in the period from \emph{Yt} to \emph{Yt+1}.
#'   \item QtPixel: \code{<int>} Pixel count that transited from the categories
#'    \emph{i} to category \emph{j} in the period from \emph{Yt} to \emph{Yt+1}.
#'   \item Interval: \code{<int>} Interval of years between the first and
#'    the last year of the period \emph{[Yt, Yt+1]}.
#'   \item yearFrom: \code{<int>} The year that the change comes from \emph{[Yt]}
#'   \item yearTo: \code{<int>} The year that the change goes for \emph{[Yt+1]}
#'   }}
#'   \item{lulc_Onstep}{\code{<tibble>} Contingency table for the entire analysed
#'    period \emph{[Yt1, YT]}, containing
#'   8 columns identical with \code{lulc_Mulstistep}}.
#'   \item{tb_legend}{\code{<tibble>} A table of the pixel value, his name and
#'   color containing 3 columns:
#'   \enumerate{
#'   \item categoryValue: \code{<int>} the pixel value of the LUC category.
#'   \item categoryName: \code{<fct>} randomly created string associated with a
#'   given pixel value of a LUC category.
#'   \item color: \code{<chr>} random color associated with the given pixel value
#'    of a LUC category.
#'   }}
#'   \item{totalArea}{\code{<tibble>} A table with the total area of the study
#'   area containing 2 columns:
#'   \enumerate{
#'   \item area_km2: \code{<dbl>} The total area in square kilometers.
#'   \item QtPixel: \code{<int>} The total area in pixel counts}}.
#'   \item{totalInterval}{\code{<int>} Total interval of the analysed time
#'   series in years}.
#'   }
#'
#' @keywords datasets
#'
#'
#' @source Embrapa Pantanal, Instituto SOS Pantanal, WWF-Brasil (2015).
"SL_2002_2014"

#'
#' Class Interval
#'
#' A S4 class for the Interval level result of an Intensity analysis.  Can be
#' plotted with the plot method \code{\link{plot}}.
#'
#' @slot intervalData tibble. A table with the results of an Intensity analysis
#' at the Interval level (\emph{St} and \emph{U} values).
#'
#' @details The slot \code{intervalData} receives a table containing 4 columns
#' in the following format:
#'    \enumerate{
#'    \item Period: \code{<fct>}. The period of interest \emph{[Yt, Yt+1]}.
#'    \item PercentChange: \code{<dbl>}. Changed area on the Interval level (\%).
#'    \item St: \code{<dbl>}. Annual intensity of change for a time
#'    period [Yt, Yt+1].
#'    \item U: \code{<dbl>}. Uniform intensity for a LUC category change in a time
#'    period of interest.
#'    }
#' @export
#' @aliases Interval
#' @exportClass Interval
#' @rdname Interval-class
#' @import methods
#'
setClass(
  Class = "Interval",
  slots = c(intervalData = "tbl_df"),
  validity = function(object) {
    if (!("tbl_df" %in% class(object@intervalData) & (ncol(object@intervalData) == 4))) {
      stop("The data has to be a `tibble` and must have 4 columns")
    }
    TRUE
  }
)


#' Class Category
#'
#' A S4 class for the Category level result of an Intensity analysis.  Can be
#' plotted with the plot method \code{\link{plot}}.
#'
#' @slot lookupcolor The colors (character vector) associated with the LUC legend items.
#' @slot categoryData tibble. A table of Category level's results (gain (\emph{Gtj})
#' or loss (\emph{Lti}) values).
#' @slot categoryStationarity tibble. A table containing results of a stationarity
#' test. A change is considered stationary only if the intensities for all time
#' intervals reside on one side of the uniform intensity, i.e are smaller or bigger
#' than the uniform rate over the whole period.
#'
#' @details The slots \code{categoryData} and \code{categoryStationarity} can receive
#'    tables for "Gain" or "Loss" in the following format:
#' \enumerate{
#'
#' \item Gain
#'
#'  \itemize{
#'
#'  \item \code{categoryData}: \code{<tibble>}. A table containing 6 columns:
#'  \enumerate{
#'    \item Period: \code{<fct>}. The period \emph{[Yt, Yt+1]}.
#'    \item To: \code{<fct>}. A LUC category \emph{j}.
#'    \item Interval: \code{<int>}. Duration of the period \emph{[Yt, Yt+1]} in years.
#'    \item GG_km2/GG_pixel: \code{<dbl>/<int>}. Area of gross gain of category
#'    \emph{j} during \emph{[Yt, Yt+1]}.
#'    \item Gtj: \code{<dbl>}. Annual intensity of gross gain of category \emph{j}
#'    for time interval \emph{[Yt, Yt+1]}.
#'    \item St: \code{<dbl>}. Annual intensity of change for time interval \emph{[Yt, Yt+1]}.
#'    }
#'  \item categoryStationarity:  \code{<tibble>}. A table with the results of a
#'  stationarity test of the gain of the categories on the Category level, containing 5 columns:
#'  \enumerate{
#'    \item To: \code{<fct>}. A category of interest \emph{j}.
#'    \item gain: \code{<int>}. Number of times a category had gains during all
#'    time intervals \emph{[Y1, YT]}.
#'    \item N: \code{<int>}. Total number of evaluated time points (T).
#'    \item Stationarity: \code{<chr>}. \emph{Active Gain} or \emph{Dormant Gain}.
#'    \item Test: \code{<chr>}. \emph{Y} if stationarity was detected and \emph{N} if not.
#'    }
#'    }
#'
#' \item Loss
#'
#'  \itemize{
#'
#'  \item \code{categoryData}: \code{<tibble>}. A table containing 6 columns:
#'  \enumerate{
#'    \item Period: \code{<fct>}. The period \emph{[Yt, Yt+1]}.
#'    \item From: \code{<fct>}. A LUC category \emph{i}.
#'    \item Interval: \code{<int>}. Duration of the period \emph{[Yt, Yt+1]} in years.
#'    \item GG_km2/GG_pixel: \code{<dbl>/<int>}. Area of gross loss of category
#'    \emph{i} during \emph{[Yt, Yt+1]}.
#'    \item Lti: \code{<dbl>}. Annual intensity of gross loss of category \emph{i}
#'    for time interval \emph{[Yt, Yt+1]}.
#'    \item STt: \code{<dbl>}. Annual intensity of change for time interval \emph{[Yt, Yt+1]}.
#'    }
#'   \item categoryStationarity: \code{<tibble>}. A table of stationarity test
#'   over the loss of the categories in the Category level, containing 5 columns:
#'  \enumerate{
#'    \item From: \code{<fct>}. A category of interest \emph{i}.
#'    \item loss: \code{<int>}. Number of times a category had losses during all
#'    time intervals \emph{[Y1, YT]}.
#'    \item N: \code{<int>}. Total number of evaluated time points (T).
#'    \item Stationarity: \code{<chr>}. \emph{Active Loss} or \emph{Dormant Loss}.
#'    \item Test: \code{<chr>}. \emph{Y} if stationarity was detected and \emph{N} if not.
#'    }
#'    }
#'
#'    }
#'
#'
#' @aliases Category
#' @export
#' @exportClass Category
#' @rdname Category-class
#'
setClass(
  Class = "Category",
  slots = c(
    lookupcolor = "character",
    categoryData = "tbl_df",
    categoryStationarity = "tbl_df"
  ),
  validity = function(object) {
    if (!("tbl_df" %in% class(object@categoryData) & (ncol(object@categoryData) == 6))) {
      stop("The `categoryData` data has to be a `tibble` and must have 5 columns")
    } else if (!("tbl_df" %in% class(object@categoryStationarity) & (ncol(object@categoryStationarity) == 5))) {
      stop("The `categoryStationarity` data has to be a `tibble` and must have 5 columns")
    } else if (!is.character(object@lookupcolor) & length(names(object@lookupcolor) == 0)) {
      stop("The `lookupcolor` has to be an object of type `character` with name attributes")
    }

    TRUE
  }
)


#' Class Transition
#'
#' A S4 class for the Transition level result of an Intensity analysis. Can be
#' plotted with the plot method \code{\link{plot}}.
#'
#' @slot lookupcolor The colors (character vector) associated with the LUC legend items.
#' @slot transitionData tibble. A table of Transition level's results (gain n (\emph{Rtin} & \emph{Wtn})
#'    or loss m (\emph{Qtmj} & \emph{Vtm}) values).
#' @slot transitionStationarity tibble. A table containing results of a stationarity
#'  test. A change is considered stationary only if the intensities for all time
#'  intervals reside on one side of the uniform intensity, i.e are smaller or bigger
#'  than the uniform rate over the whole period.
#'
#'
#'
#' @details The slots \code{transitionData} and \code{transitionStationarity} can
#' receive tables for "Gain of category n" or "Loss of category m" in the following
#' format:
#' \enumerate{
#'
#' \item Gain of category n:
#'
#'  \itemize{
#'
#'   \item \code{transitionData}: \code{<tibble>}. A table with 7 columns:
#'   \enumerate{
#'    \item Period: \code{<fct>}. The period \emph{[Yt, Yt+1]}.
#'    \item From: \code{<fct>}. A category i.
#'    \item To: \code{<fct>}. The gaining category in the transition of interest \emph{(n)}.
#'    \item Interval: \code{<int>}. Duration of the period \emph{[Yt, Yt+1]}.
#'    \item T_i2n_km2/T_i2n_pixel: \code{<dbl>}. Area with transition from category
#'    i to category n during time interval \emph{[Yt, Yt+1]} where \emph{i}
#'    \code{is not equal to} \emph{n}.
#'    \item Rtin: \code{<dbl>}. Annual intensity of transition from category i to
#'    category n
#'    during time interval \emph{[Yt, Yt+1]} where \emph{i}  \code{is not equal to} \emph{n}.
#'    \item Wtn: \code{<dbl>}. Value of the uniform intensity of the transition
#'    to category n from all non-n categories at time Yt during time interval \emph{[Yt, Yt+1]}.
#'    }
#'  \item transitionStationarity: \code{<tibble>}. A table containing results of
#'  a stationarity test over the gain on \emph{category n} containing 5 columns:
#'  \enumerate{
#'    \item From: \code{<fct>}. The losing category in the transition of interest
#'    to the category n.
#'    \item loss: \code{<int>}. Number of times the category had losses to the
#'    category n.
#'    \item N: \code{<int>}. Total number of transitions to be considered as stationary (T).
#'    \item Stationarity: \code{<chr>}. \emph{targeted by} or \emph{avoided by}
#'    the category \code{n}.
#'    \item Test: \code{<chr>}. \emph{Y} for stationarity detected and \emph{N}
#'    when not.
#'    }
#'    }
#'
#' \item Loss of category m:
#'
#'  \itemize{
#'
#'   \item \code{transitionData}: \code{<tibble>}. A table with 7 columns:
#'   \enumerate{
#'    \item Period: \code{<fct>}. The period \emph{[Yt, Yt+1]}.
#'    \item To: \code{<fct>}. A category \emph{j}.
#'    \item From: \code{<fct>}. The losing category in the transition of interest (m).
#'    \item Interval: \code{<dbl>}. Duration of the period \emph{[Yt, Yt+1]}.
#'    \item T_m2j_km2/T_m2j_pixel: \code{<dbl>}. Area with transition from category
#'     \emph{m} to category \emph{j} during time interval \emph{[Yt, Yt+1]}
#'    where \emph{j}  \code{is not equal to} \emph{m}.
#'    \item Qtmj: \code{<dbl>}. Annual intensity of transition from category \emph{m} to
#'    category \emph{j} during time interval \emph{[Yt, Yt+1]}
#'    where \emph{j}  \code{is not equal to} \emph{m}.
#'    \item Vtm: \code{<dbl>}. Value of the uniform intensity of the transition
#'    from category \emph{m} to all \emph{non-m} categories at time \out{Y<sub>t+1</sub>}
#'    during time interval \emph{[Yt, Yt+1]}.
#'    }
#'  \item transitionStationarity: \code{<tibble>}. A table containing results of
#'  a stationarity test
#'  over the loss of \emph{category m} containing 5 columns:
#'  \enumerate{
#'    \item To: \code{<fct>}. The gaining category in the transition of interest
#'    from the category m.
#'    \item gain: \code{<int>}. Number of times the category had gains from the
#'    category m.
#'    \item N: \code{<int>}. Total number of transitions to be considered as stationary (T).
#'    \item Stationarity: \code{<chr>}. \emph{targeted} or \emph{avoided} the category \code{m}.
#'    \item Test: \code{<chr>}. \emph{Y} for stationarity detected and \emph{N} when not.
#'    }
#'    }
#'
#' }
#' @aliases Transition
#' @export
#' @exportClass Transition
#' @rdname Transition-class
#'
setClass(
  Class = "Transition",
  slots = c(
    lookupcolor = "character",
    transitionData = "tbl_df",
    transitionStationarity = "tbl_df"
  ),
  validity = function(object) {
    if (!("tbl_df" %in% class(object@transitionData) & (ncol(object@transitionData) == 7))) {
      stop("The `transitionData` data has to be a `tibble` and must have 7 columns")
    } else if (!("tbl_df" %in% class(object@transitionStationarity) & (ncol(object@transitionStationarity) == 5))) {
      stop("The `categoryStationarity` data has to be a `tibble` and must have 5 columns")
    } else if (!is.character(object@lookupcolor) & length(names(object@lookupcolor) == 0)) {
      stop("The `lookupcolor` has to be an object of type `character` with name attributes")
    }
    TRUE
  }
)

utils::globalVariables(c("Type", "St", "U"))
#' @include intensityAnalysis.R
NULL

#' Methods for function \code{plot} in package \pkg{OpenLand}
#'
#' Plot \code{Intensity} objects based on Intensity Analysis output.
#'
#' @param x An intensity object generated by \code{\link{intensityAnalysis}}.
#' @param y ignored.
#' @param labels character. Left and right axis titles(caption).
#' @param title character. Main title.
#' @param labs character. The lateral legend.
#' @param marginplot numeric. Adjustment of the origins of left and right part of
#' the plots.
#' @param leg_curv numeric. x and y values that control the arrow size and position
#' pointing to the Uniform Intensity vertical line.
#' @param color_bar character. Colors defined for the fast, slow and area bars
#' (only for an \code{\linkS4class{Interval}} object).
#' @param fontsize_ui numeric. Fontsize of the uniform intensity percent in the plot.
#' @param \dots additional arguments for theme parameters from ggplot2, see
#' \code{\link[ggplot2]{theme}}.
#'
#' @return An intensity graph
#'
#'
#' @import ggplot2
#' @importFrom gridExtra grid.arrange
#' @importFrom grid textGrob gpar
#'
#' @keywords methods plot
#' @docType methods
#' @rdname plot
#' @aliases plot,ANY,ANY-method plot,Interval,ANY-method plot,Category,ANY-method plot,Transition,ANY-method
#' @export
#'
#'
methods::setGeneric(name = "plot", def = function(x, y, ...) {
  standardGeneric("plot")
})


#' @param Interval The class.
#' @docType methods
#' @rdname plot
#' @export
#' @aliases plot,Interval,ANY-method
#'
#'
methods::setMethod(
  f = "plot",
  signature("Interval", "ANY"),
  definition = function(x,
                        y,
                        labels = c(
                          leftlabel = "Interval Change Area (percent of map)",
                          rightlabel = "Annual Change Area (percent of map)"
                        ),
                        title = NA,
                        labs = c(type = "Changes", ur = "Uniform Intensity"),
                        marginplot = c(lh = -10, rh = 0),
                        leg_curv = c(x = 0.1, y = 0.1),
                        color_bar = c(
                          fast = "#B22222",
                          slow = "#006400",
                          area = "gray40"
                        ),
                        fontsize_ui = 10,
                        ...) {
    dataset <-
      x$intervalData %>% dplyr::mutate(Type = ifelse(St > U, "Fast", "Slow"))


    GL01_taxa <-
      dataset %>% ggplot(aes(factor(dataset[[1]], levels = rev(levels(dataset[[1]]))), dataset[[3]])) +
      geom_bar(
        aes(fill = Type),
        stat = "identity",
        position = "dodge",
        width = .5
      ) +
      geom_hline(aes(yintercept = dataset[[4]], color = "U"),
        linetype = 5,
        linewidth = .5
      ) +
      geom_hline(aes(yintercept = 0), linewidth = .01) +
      scale_fill_manual(values = c(color_bar[[1]], color_bar[[2]])) +
      ylab(NULL) +
      scale_color_manual(values = "black") +
      labs(fill = labs[[1]], color = labs[[2]]) +
      scale_y_continuous(expand = expansion(mult = c(0, .01))) +
      scale_x_discrete(expand = expansion(mult = c(0.06, 0.06))) +
      guides(
        fill = guide_legend(order = 1),
        color = guide_legend(order = 2)
      ) +
      coord_flip() +
      geom_curve(
        aes(
          x = length(unique(dataset[[1]])) / 2,
          y = dataset[[4]],
          xend = (length(unique(dataset[[1]])) / 2) + leg_curv[[2]],
          yend = dataset[[4]] + leg_curv[[1]]
        ),
        linewidth = .6,
        curvature = .1,
        arrow = arrow(length = unit(2, "mm"), ends = "first")
      ) +
      geom_text(
        aes(
          x = (length(unique(
            dataset[[1]]
          )) / 2) + leg_curv[[2]],
          y = dataset[[4]] + leg_curv[[1]]
        ),
        label = paste(round(dataset[[4]], 2), "%"),
        colour = "black",
        fontface = c("plain", "bold", "italic", "bold.italic")[1],
        family = c("sans", "serif", "mono")[1],
        size = fontsize_ui / .pt,
        nudge_y = 1 / 100,
        hjust = "left"
      ) +
      theme(
        axis.title.y = element_blank(),
        axis.text.y = element_text(margin = margin(r = 8)),
        axis.ticks.length.y = unit(2, "pt"),
        legend.position = "right",
        legend.direction = "vertical",
        plot.margin = unit(c(
          t = 0,
          r = 0,
          b = 0,
          l = -(marginplot[[2]])
        ), "pt"), ...
      )

    # Area---------------
    GL01_area <-
      dataset %>% ggplot(aes(factor(dataset[[1]], levels = rev(levels(dataset[[1]]))), dataset[[2]])) +
      geom_bar(
        stat = "identity",
        position = "dodge",
        width = .5,
        fill = color_bar[[3]]
      ) +
      coord_flip() +
      xlab(expression(paste("Periodo de tempo [ ", Y[t], ",", Y[t + 1], "]"))) +
      ylab(NULL) +
      geom_hline(aes(yintercept = 0), linewidth = .01) +
      scale_y_reverse(expand = expansion(mult = c(0.01, 0))) +
      scale_x_discrete(position = "top", expand = expansion(mult = c(0.06, 0.06))) +
      theme(
        axis.title.y = element_blank(),
        axis.ticks.length.y = unit(2, "pt"),
        axis.text.y = element_blank(),
        plot.margin = unit(c(
          t = 0,
          r = -(marginplot[[1]]),
          b = 0,
          l = 0
        ), "pt"), ...
      )


    format_lab <- function(x, font = 1, size = 11) {
      grid::textGrob(x, gp = grid::gpar(fontface = font, fontsize = size))
    }

    if (!is.na(title)) {
      title_lab <- format_lab(title)
      left_lab <- format_lab(labels[[1]])
      right_lab <- format_lab(labels[[2]])

      my_layout <-
        matrix(
          c(rep(1, 6), NA, rep(rep(2:3, c(
            3, 4
          )), 20), rep(4:5, c(3, 3)), NA),
          ncol = 7,
          byrow = TRUE
        )

      gridExtra::grid.arrange(title_lab,
        GL01_area,
        GL01_taxa,
        left_lab,
        right_lab,
        layout_matrix = my_layout
      )
    } else {
      left_lab <- format_lab(labels[[1]])
      right_lab <- format_lab(labels[[2]])

      my_layout <-
        matrix(
          c(rep(rep(2:3, c(
            3, 5
          )), 20), rep(c(4, 5, NA), c(3, 4, 1))),
          ncol = 8, byrow = TRUE
        )

      gridExtra::grid.arrange(GL01_area, GL01_taxa,
        left_lab, right_lab,
        layout_matrix = my_layout
      )
    }
  }
)


#' @param Category The class.
#' @docType methods
#' @rdname plot
#' @export
#'
#' @aliases plot,Category,ANY-method
methods::setMethod(
  f = "plot",
  signature("Category", "ANY"),
  definition = function(x,
                        y,
                        labels = c(
                          leftlabel = "Annual Change Area (km2 or pixels)",
                          rightlabel = "Annual Change Intensity (percent of category)"
                        ),
                        title = NA,
                        labs = c(type = "Categories", ur = "Uniform Intensity"),
                        marginplot = c(lh = 0.5, rh = 0.5),
                        leg_curv = c(x = 0.1, y = 0.1),
                        fontsize_ui = 10,
                        ...) {
    dataset <- x$categoryData
    lookupcolor <- x$lookupcolor
    category_unique <- as.character(unique(dataset[[2]]))
    fill_values <- unname(lookupcolor[category_unique[order(match(category_unique, levels(dataset[[2]])))]])

    GL02_ganho_taxa <-
      dataset %>% ggplot(aes(factor(dataset[[2]], levels = rev(levels(dataset[[2]]))), dataset[[5]])) +
      geom_bar(aes(fill = dataset[[2]]), stat = "identity", position = "dodge") +
      facet_wrap(~ dataset[[1]], ncol = 1) +
      scale_fill_manual(values = fill_values) +
      xlab(NULL) +
      ylab(NULL) +
      geom_hline(aes(yintercept = 0), linewidth = .3) +
      geom_hline(aes(yintercept = dataset[[6]], color = names(dataset)[[6]]),
        linetype = 5,
        linewidth = .3
      ) +
      scale_color_manual(values = "black") +
      coord_flip() +
      labs(fill = labs[[1]], colour = labs[[2]]) +
      scale_y_continuous(expand = expansion(mult = c(0, .01))) +
      guides(
        fill = guide_legend(order = 1),
        color = guide_legend(order = 2)
      ) +
      geom_curve(
        aes(
          x = length(unique(dataset[[2]])) / 2,
          y = dataset[[6]],
          xend = (length(unique(dataset[[2]])) / 2) + leg_curv[[2]],
          yend = dataset[[6]] + leg_curv[[1]]
        ),
        curvature = .1,
        arrow = arrow(length = unit(2, "mm"), ends = "first")
      ) +
      geom_text(
        aes(
          x = (length(unique(
            dataset[[2]]
          )) / 2) + leg_curv[[2]],
          y = dataset[[6]] + leg_curv[[1]]
        ),
        label = paste(round(dataset[[6]], 2), "%"),
        colour = "black",
        fontface = c("plain", "bold", "italic", "bold.italic")[1],
        family = c("sans", "serif", "mono")[1],
        size = fontsize_ui / .pt,
        nudge_y = 1 / 100,
        hjust = "left"
      ) +
      theme(
        axis.text.y = element_blank(),
        axis.title.x = element_text(size = 13, face = "plain"),
        axis.ticks.length.y = unit(0, "pt"),
        legend.position = "right",
        legend.direction = "vertical",
        plot.margin = unit(c(
          t = 0,
          r = 0,
          b = 0,
          l = -(marginplot[[2]])
        ), "pt"), ...
      )

    # Area ----
    GL02_ganho_area <-
      dataset %>% ggplot(aes(factor(dataset[[2]], levels = rev(levels(dataset[[2]]))), dataset[[4]])) +
      geom_bar(aes(fill = dataset[[2]]), stat = "identity", position = "dodge") +
      facet_wrap(~ dataset[[1]], ncol = 1) +
      scale_fill_manual(values = fill_values) +
      xlab(NULL) +
      ylab(NULL) +
      geom_hline(aes(yintercept = 0), linewidth = .3) +
      coord_flip() +
      labs(fill = "Categories") +
      scale_y_reverse(expand = expansion(mult = c(0.01, 0))) +
      scale_x_discrete(position = "top") +
      theme(
        axis.ticks.length.y = unit(0, "pt"),
        axis.title.x = element_text(size = 12, face = "plain"),
        axis.text.y = element_blank(),
        legend.position = "none",
        plot.margin = unit(c(
          t = 0,
          r = -(marginplot[[1]]),
          b = 0,
          l = 0
        ), "pt"), ...
      )

    format_lab <- function(x, font = 1, size = 11) {
      grid::textGrob(x, gp = grid::gpar(fontface = font, fontsize = size))
    }

    if (!is.na(title)) {
      title_lab <- format_lab(title)
      left_lab <- format_lab(labels[[1]])
      right_lab <- format_lab(labels[[2]])

      my_layout <-
        matrix(
          c(rep(1, 6), NA, rep(rep(2:3, c(
            3, 4
          )), 20), rep(4:5, c(3, 3)), NA),
          ncol = 7,
          byrow = TRUE
        )

      gridExtra::grid.arrange(
        title_lab,
        GL02_ganho_area,
        GL02_ganho_taxa,
        left_lab,
        right_lab,
        layout_matrix = my_layout
      )
    } else {
      left_lab <- format_lab(labels[[1]])
      right_lab <- format_lab(labels[[2]])

      my_layout <-
        matrix(
          c(rep(rep(2:3, c(
            3, 4
          )), 20), rep(c(4, 5, NA), c(3, 3, 1))),
          ncol = 7, byrow = TRUE
        )

      gridExtra::grid.arrange(GL02_ganho_area,
        GL02_ganho_taxa,
        left_lab,
        right_lab,
        layout_matrix = my_layout
      )
    }
  }
)


#' @param Transition The class.
#' @docType methods
#' @rdname plot
#' @export
#'
#' @aliases plot,Transition,ANY-method
methods::setMethod(
  f = "plot",
  signature("Transition", "ANY"),
  definition = function(x,
                        y,
                        labels = c(
                          leftlabel = "Annual Transition Area (km2 or pixels)",
                          rightlabel = "Annual Transition Intensity (percent of category)"
                        ),
                        title = NA,
                        labs = c(type = "Categories", ur = "Uniform Intensity"),
                        marginplot = c(lh = 0.5, rh = 0.5),
                        leg_curv = c(x = 0.1, y = 0.1),
                        fontsize_ui = 10,
                        ...) {
    dataset <- x$transitionData
    lookupcolor <- x$lookupcolor
    category_unique <- as.character(unique(dataset[[2]]))
    fill_values <- unname(lookupcolor[category_unique[order(match(category_unique, levels(dataset[[2]])))]])

    GL03_ganho_taxa <-
      dataset %>% ggplot(aes(factor(dataset[[2]], levels = rev(levels(dataset[[2]]))), dataset[[6]])) +
      geom_bar(aes(fill = dataset[[2]]), stat = "identity", position = "dodge") +
      facet_wrap(~ dataset[[1]], ncol = 1) +
      scale_fill_manual(values = fill_values) +
      xlab(NULL) +
      ylab(NULL) +
      geom_hline(aes(yintercept = 0), linewidth = .3) +
      geom_hline(aes(yintercept = dataset[[7]], color = names(dataset)[[7]]),
        linetype = 5,
        linewidth = .3
      ) +
      scale_color_manual(values = "black") +
      coord_flip() +
      labs(fill = labs[[1]], colour = labs[[2]]) +
      scale_y_continuous(expand = expansion(mult = c(0, .01))) +
      guides(
        fill = guide_legend(order = 1),
        color = guide_legend(order = 2)
      ) +
      geom_curve(
        aes(
          x = length(unique(dataset[[2]])) / 2,
          y = dataset[[7]],
          xend = (length(unique(dataset[[2]])) / 2) + leg_curv[[2]],
          yend = dataset[[7]] + leg_curv[[1]]
        ),
        curvature = .1,
        arrow = arrow(length = unit(2, "mm"), ends = "first")
      ) +
      geom_text(
        aes(
          x = (length(unique(
            dataset[[2]]
          )) / 2) + leg_curv[[2]],
          y = dataset[[7]] + leg_curv[[1]]
        ),
        label = paste(round(dataset[[7]], 2), "%"),
        colour = "black",
        fontface = c("plain", "bold", "italic", "bold.italic")[1],
        family = c("sans", "serif", "mono")[1],
        size = fontsize_ui / .pt,
        nudge_y = 1 / 100,
        hjust = "left"
      ) +
      theme(
        axis.text.y = element_blank(),
        axis.title.x = element_text(size = 13, face = "plain"),
        axis.ticks.length.y = unit(0, "pt"),
        legend.position = "right",
        legend.direction = "vertical",
        plot.margin = unit(c(
          t = 0,
          r = 0,
          b = 0,
          l = -(marginplot[[2]])
        ), "pt"), ...
      )

    # Area ----
    GL03_ganho_area <- dataset %>%
      ggplot(aes(factor(dataset[[2]], levels = rev(levels(dataset[[2]]))), dataset[[5]])) +
      geom_bar(aes(fill = dataset[[2]]), stat = "identity", position = "dodge") +
      facet_wrap(~ dataset[[1]], ncol = 1) +
      scale_fill_manual(values = fill_values) +
      xlab(NULL) +
      ylab(NULL) +
      geom_hline(aes(yintercept = 0), linewidth = .3) +
      coord_flip() +
      labs(fill = "Categories") +
      scale_y_reverse(expand = expansion(mult = c(0.01, 0))) +
      scale_x_discrete(position = "top") +
      theme(
        axis.ticks.length.y = unit(0, "pt"),
        axis.title.x = element_text(size = 12, face = "plain"),
        axis.text.y = element_blank(),
        legend.position = "none",
        plot.margin = unit(c(
          t = 0,
          r = -(marginplot[[1]]),
          b = 0,
          l = 0
        ), "pt"), ...
      )


    format_lab <- function(x, font = 1, size = 11) {
      grid::textGrob(x, gp = grid::gpar(fontface = font, fontsize = size))
    }

    if (!is.na(title)) {
      title_lab <- format_lab(title)
      left_lab <- format_lab(labels[[1]])
      right_lab <- format_lab(labels[[2]])

      my_layout <-
        matrix(
          c(rep(1, 6), NA, rep(rep(2:3, c(
            3, 4
          )), 20), rep(4:5, c(3, 3)), NA),
          ncol = 7,
          byrow = TRUE
        )

      gridExtra::grid.arrange(
        title_lab,
        GL03_ganho_area,
        GL03_ganho_taxa,
        left_lab,
        right_lab,
        layout_matrix = my_layout
      )
    } else {
      left_lab <- format_lab(labels[[1]])
      right_lab <- format_lab(labels[[2]])

      my_layout <-
        matrix(
          c(rep(rep(2:3, c(
            3, 4
          )), 20), rep(c(4, 5, NA), c(3, 3, 1))),
          ncol = 7, byrow = TRUE
        )

      gridExtra::grid.arrange(GL03_ganho_area,
        GL03_ganho_taxa,
        left_lab,
        right_lab,
        layout_matrix = my_layout
      )
    }
  }
)

utils::globalVariables(c(
  "From", "To", "target", "km2", "Year",
  "QtPixel", "yearFrom", "yearTo", "name",
  "colorFrom", "colorTo", "lulc", "area",
  "Category", "Years", "flow_id",
  "geom_flow", "geom_stratum", "area_gross"
))
#' @include plotMethods.R
NULL

#' Area of LUC categories at time points
#'
#'
#' A grouped barplot representing the areas of LUC categories at each time point
#' of the analysed period.
#'
#' @param dataset A table of the multi step transitions (\code{lulc_Multistep})
#' generated by \code{\link{contingencyTable}}.
#' @param legendtable A table containing the LUC legend items and their respective
#' color (\code{tb_legend}).
#' @param title character. The title of the plot.
#' @param caption character. The caption of the plot.
#' @param xlab character. Label for the x axis.
#' @param ylab character. Label for the y axis.
#' @param area_km2 logical. If TRUE the change is computed in km2, if FALSE in
#' pixel counts.
#' @param \dots additional themes parameters, see \code{\link[ggplot2]{theme}}.
#'
#'
#' @seealso \code{ggplot2::\link[ggplot2]{theme}}
#'
#' @return a barplot
#' @export
#'
#' @importFrom graphics par legend text
#'
#' @examples
#'
#' # editing the category names
#'
#' SL_2002_2014$tb_legend$categoryName <- factor(
#'   c(
#'     "Ap", "FF", "SA", "SG", "aa", "SF",
#'     "Agua", "Iu", "Ac", "R", "Im"
#'   ),
#'   levels = c(
#'     "FF", "SF", "SA", "SG", "aa", "Ap",
#'     "Ac", "Im", "Iu", "Agua", "R"
#'   )
#' )
#'
#  # add the color by the same order of the legend factor
#' SL_2002_2014$tb_legend$color <- c(
#'   "#FFE4B5", "#228B22", "#00FF00", "#CAFF70",
#'   "#EE6363", "#00CD00", "#436EEE", "#FFAEB9",
#'   "#FFA54F", "#68228B", "#636363"
#' )
#' # the plot
#' barplotLand(
#'   dataset = SL_2002_2014$lulc_Multistep,
#'   legendtable = SL_2002_2014$tb_legend,
#'   area_km2 = TRUE
#' )
#'
barplotLand <-
  function(dataset,
           legendtable,
           title = NULL,
           caption = "LUC Categories",
           xlab = "Year",
           ylab = "Area (km2 or pixel)",
           area_km2 = TRUE, ...) {
    datachange <- dataset %>%
      left_join(legendtable, by = c("From" = "categoryValue")) %>%
      left_join(legendtable, by = c("To" = "categoryValue")) %>%
      dplyr::select(-c(From, To)) %>%
      rename(
        "From" = "categoryName.x",
        "To" = "categoryName.y",
        "colorFrom" = "color.x",
        "colorTo" = "color.y"
      )

    areaif <- ifelse(isTRUE(area_km2), "km2", "QtPixel")

    datanual <-
      datachange %>%
      group_by(yearTo, To) %>%
      summarise(area = sum(!!as.name(areaif))) %>%
      rename("Year" = "yearTo", "lulc" = "To") %>%
      rbind(
        datachange[datachange$yearFrom == first(datachange$yearFrom), ] %>%
          group_by(yearFrom, From) %>% # capturing the first year change
          summarise(area = sum(!!as.name(areaif))) %>%
          rename("Year" = "yearFrom", "lulc" = "From")
      )

    ggplot(data = datanual, aes(as.character(Year), area)) +
      geom_bar(aes(fill = lulc), stat = "identity", position = "dodge") +
      scale_fill_manual(values = legendtable$color[order(legendtable$categoryName)]) +
      labs(fill = caption) +
      xlab(xlab) +
      ylab(ylab) +
      ggtitle(title) +
      theme(
        plot.title = element_text(hjust = .5),
        ...
      )
  }


#' One step transitions (Chord diagram)
#'
#'
#' A circlize plot representing the one step transitions between two times point
#' of interest.
#'
#' @param dataset A table of the one step transition (\code{lulc_OneStep}) generated
#' by \code{\link{contingencyTable}}.
#' @param legendtable A table containing the LUC legend items and their respective
#' color (\code{tb_legend}).
#' @param legposition numeric. A vector containing the `x` and `y` values for the
#' position of the legend. (see \code{\link[graphics]{legend}}).
#' @param legtitle character. The title of the legend.
#' @param sectorcol character. The color of the external sector containing the years
#' of compared time points.
#' @param area_km2 logical. If TRUE the change is computed in km2, if FALSE in
#' pixel counts.
#' @param legendsize numeric. Font size of the legend. (see "cex" in \code{\link[graphics]{legend}}).
#' @param y.intersp numeric. character interspacing factor for vertical (y)
#' spacing in the legend.
#' @param x.margin numeric vector ensuring additional space (blank area) on the
#' left or right of the circle for the legend, by default it is c(-1, 1). (see
#' "canvas.xlim" in \code{\link[circlize]{circos.par}})
#'
#' @return A Chord Diagram
#' @export
#'
#' @examples
#'
#' # editing the category names
#'
#' SL_2002_2014$tb_legend$categoryName <- factor(
#'   c(
#'     "Ap", "FF", "SA", "SG", "aa", "SF",
#'     "Agua", "Iu", "Ac", "R", "Im"
#'   ),
#'   levels = c(
#'     "FF", "SF", "SA", "SG", "aa", "Ap",
#'     "Ac", "Im", "Iu", "Agua", "R"
#'   )
#' )
#'
#  # add the color by the same order of the legend factor
#' SL_2002_2014$tb_legend$color <- c(
#'   "#FFE4B5", "#228B22", "#00FF00", "#CAFF70",
#'   "#EE6363", "#00CD00", "#436EEE", "#FFAEB9",
#'   "#FFA54F", "#68228B", "#636363"
#' )
#'
#' # the plot
#' chordDiagramLand(
#'   dataset = SL_2002_2014$lulc_Onestep,
#'   legendtable = SL_2002_2014$tb_legend
#' )
#'
chordDiagramLand <-
  function(dataset,
           legendtable,
           legposition = c(x = -1.3, y = 0),
           legtitle = "Categories",
           sectorcol = "gray80",
           area_km2 = TRUE,
           legendsize = 1,
           y.intersp = 1,
           x.margin = c(-1, 1)) {
    circle_data <- dataset %>%
      left_join(legendtable, by = c("From" = "categoryValue")) %>%
      left_join(legendtable, by = c("To" = "categoryValue")) %>%
      dplyr::select(-c(From, To)) %>%
      rename(
        "From" = "categoryName.x",
        "To" = "categoryName.y",
        "colorFrom" = "color.x",
        "colorTo" = "color.y"
      ) %>%
      tidyr::unite("source",
        c("From", "yearFrom"),
        sep = "-",
        remove = FALSE
      ) %>%
      tidyr::unite("target",
        c("To", "yearTo"),
        sep = "-",
        remove = FALSE
      ) %>%
      dplyr::select(
        source,
        target,
        From,
        To,
        km2,
        QtPixel,
        yearFrom,
        yearTo,
        colorFrom,
        colorTo
      )

    # input for the circlize function
    onestepcircle <-
      circle_data[order(circle_data$From), ][, c("source", "target", "km2", "QtPixel")]

    # seting the grid.color parameter automatic
    grid_a <- unique(circle_data$colorFrom[order(circle_data$From)])
    names(grid_a) <-
      unique(circle_data$source[order(circle_data$From)])

    grid_b <- unique(circle_data$colorTo[order(circle_data$To)])
    names(grid_b) <- unique(circle_data$target[order(circle_data$To)])

    grid.col <- c(grid_a, grid_b)


    # first and last year
    ano01 <- unique(onestepcircle$source)
    ano02 <- unique(onestepcircle$target)

    # km2 or pixel
    if (isTRUE(area_km2)) {
      onestepcircle <- onestepcircle[c(1, 2, 3)]
    } else {
      onestepcircle <- onestepcircle[c(1, 2, 4)]
    }

    old.par <- graphics::par(no.readonly = TRUE)
    on.exit(graphics::par(old.par))
    circlize::circos.clear()

    # parameters
    circlize::circos.par(
      start.degree = 0,
      gap.degree = 1,
      track.margin = c(-0.01, 0.015),
      points.overflow.warning = TRUE,
      "canvas.xlim" = c(x.margin[[1]], x.margin[[2]])
    )

    graphics::par(mar = rep(0, 4)) # outer part

    # the base plot
    circlize::chordDiagram(
      x = onestepcircle,
      grid.col = grid.col,
      transparency = 0.25,
      directional = 1,
      direction.type = c("arrows", "diffHeight"),
      diffHeight = -0.02,
      annotationTrack = c("name", "grid")[2],
      annotationTrackHeight = c(0.05, 0.1),
      link.arr.type = "big.arrow",
      link.sort = TRUE,
      link.decreasing = FALSE,
      link.largest.ontop = TRUE,
      preAllocateTracks = list(
        track.height = circlize::uh(5, "mm"),
        track.margin = c(circlize::uh(4, "mm"), circlize::uh(0, "mm"))
      )
    )

    # the km2 label
    for (si in circlize::get.all.sector.index()) {
      circlize::circos.axis(
        h = "top",
        labels.cex = .6,
        sector.index = si,
        track.index = 2
      )
    }

    # adding the externs arcs
    circlize::highlight.sector(
      ano01,
      track.index = 1,
      col = sectorcol,
      text = circle_data$yearFrom[1],
      cex = 0.9,
      text.col = "black",
      niceFacing = TRUE
    )
    circlize::highlight.sector(
      ano02,
      track.index = 1,
      col = sectorcol,
      text = circle_data$yearTo[1],
      cex = 0.9,
      text.col = "black",
      niceFacing = TRUE
    )

    # the legend
    graphics::legend(
      x = legposition[[1]],
      y = legposition[[2]],
      legend = levels(circle_data$From),
      pt.cex = 0,
      cex = legendsize,
      bty = "n",
      y.intersp = y.intersp,
      fill = legendtable$color[order(legendtable$categoryName)]
    )
    # the title
    graphics::text(
      x = legposition[[1]],
      y = legposition[[2]] + 0.01,
      labels = legtitle,
      pos = 4,
      adj = c(0, 1),
      font = 2,
      cex = legendsize + (legendsize * 0.3)
    )


    circlize::circos.clear()
  }


#' Net and gross changes of LUC categories
#'
#'
#' A stacked barplot showing net and gross changes of LUC categories during the
#' entire analysed time period.
#'
#'
#' @param dataset A table of the multi step transition (\code{lulc_Mulstistep})
#' generated by \code{\link{contingencyTable}}.
#' @param legendtable A table containing the LUC legend items and their respective
#' color (\code{tb_legend}).
#' @param title character. The title of the plot (optional), use \code{NULL} for
#' no title.
#' @param xlab character. Label for the x axis.
#' @param ylab character. Label for the y axis.
#' @param legend_title character. The title of the legend.
#' @param changesLabel character. Labels for the three types of changes, defaults
#' are c(GC = "Gross change", NG = "Net gain", NL = "Net loss").
#' @param color character. A vector defining the three bar colors.
#' @param area_km2  logical. If TRUE the change is computed in km2, if FALSE in
#' pixel counts.
#'
#'
#' @return A bar plot
#' @export
#'
#' @examples
#'
#' # editing the category names
#'
#' SL_2002_2014$tb_legend$categoryName <- factor(
#'   c(
#'     "Ap", "FF", "SA", "SG", "aa", "SF",
#'     "Agua", "Iu", "Ac", "R", "Im"
#'   ),
#'   levels = c(
#'     "FF", "SF", "SA", "SG", "aa", "Ap",
#'     "Ac", "Im", "Iu", "Agua", "R"
#'   )
#' )
#'
#' # the plot
#' netgrossplot(
#'   dataset = SL_2002_2014$lulc_Multistep,
#'   legendtable = SL_2002_2014$tb_legend,
#'   title = NULL,
#'   xlab = "LUC Category",
#'   changes = c(GC = "Gross changes", NG = "Net Gain", NL = "Net Loss"),
#'   color = c(GC = "gray70", NG = "#006400", NL = "#EE2C2C")
#' )
#'
netgrossplot <-
  function(dataset,
           legendtable,
           title = NULL,
           xlab = "LUC category",
           ylab = "Area (Km2)",
           legend_title = "Changes",
           changesLabel = c(GC = "Gross change", NG = "Net gain", NL = "Net loss"),
           color = c(GC = "gray70", NG = "#006400", NL = "#EE2C2C"),
           area_km2 = TRUE) {
    datachange <- (dataset %>%
      left_join(legendtable, by = c("From" = "categoryValue")) %>%
      left_join(legendtable, by = c("To" = "categoryValue")) %>%
      dplyr::select(-c(From, To)) %>%
      rename(
        "From" = "categoryName.x",
        "To" = "categoryName.y"
      ))[c(1, 2, 3, 7, 9)]

    areaif <- ifelse(isTRUE(area_km2), "km2", "QtPixel")


    lulc_gain <- datachange %>% dplyr::filter(From != To)

    lulc_loss <- lulc_gain %>%
      rename("To" = "From", "From" = "To") %>%
      mutate(km2 = -1 * km2, QtPixel = -1 * QtPixel)


    lulc_gainloss_gross <- rbind(lulc_gain, lulc_loss) %>%
      mutate(changes = ifelse(QtPixel > 0, "Gain", "Loss"))


    lulc_gainLoss_net <-
      lulc_gainloss_gross %>%
      group_by(To) %>%
      summarise(area = sum(!!as.name(areaif))) %>%
      mutate(changes = ifelse(area > 0, changesLabel[[2]], changesLabel[[3]]))

    if (isTRUE(area_km2)) {
      lulc_gainloss_gross <- lulc_gainloss_gross[c(1, 2, 4, 5, 6)]
    } else {
      lulc_gainloss_gross <- lulc_gainloss_gross[c(1, 3, 4, 5, 6)]
    }

    names(lulc_gainloss_gross) <- c("Period", "area_gross", "From", "To", "changes")
    names(color) <- unname(changesLabel[c("GC", "NG", "NL")]) # pairing the legend with the color


    ggplot(data = lulc_gainloss_gross, aes(To, area_gross)) +
      geom_bar(stat = "identity", width = 0.5, aes(fill = changesLabel[[1]])) +
      geom_bar(
        aes(x = To, y = area, fill = changes),
        data = lulc_gainLoss_net,
        stat = "identity",
        width = 0.4,
        inherit.aes = FALSE
      ) +
      geom_segment(
        data = lulc_gainLoss_net,
        aes(
          x = as.numeric(To) - 0.3,
          y = area,
          xend = as.numeric(To) + 0.3,
          yend = area
        )
      ) +
      scale_fill_manual(values = color) +
      labs(fill = legend_title) +
      geom_hline(yintercept = 0, linewidth = .3) +
      xlab(xlab) +
      ylab(ylab) +
      ggtitle(title) +
      theme(plot.title = element_text(hjust = .5))
  }


#' Sankey diagram of LUC transitions (one or multistep)
#'
#' A sankey showing the one or multi step LUC transitions during the analysed period.
#'
#' @param dataset A table of the multi step (\code{lulc_Mulstistep}).
#' or one step transitions (\code{lulc_OneStep}) generated by \code{\link{contingencyTable}}.
#' @param legendtable A table containing the LUC legend items and their respective
#' color (\code{tb_legend}).
#' @param iterations numeric. Number of iterations in the diagram layout for
#' computation of the depth (y-position) of each node. See \code{\link[networkD3]{sankeyNetwork}}.
#'
#' @seealso \code{\link[networkD3]{sankeyNetwork}}
#'
#'
#' @return A sankey diagram
#' @export
#'
#' @examples
#'
#' # editing the category names
#'
#' SL_2002_2014$tb_legend$categoryName <- factor(
#'   c(
#'     "Ap", "FF", "SA", "SG", "aa", "SF",
#'     "Agua", "Iu", "Ac", "R", "Im"
#'   ),
#'   levels = c(
#'     "FF", "SF", "SA", "SG", "aa", "Ap",
#'     "Ac", "Im", "Iu", "Agua", "R"
#'   )
#' )
#'
#  # add the color by the same order of the legend factor
#' SL_2002_2014$tb_legend$color <- c(
#'   "#FFE4B5", "#228B22", "#00FF00", "#CAFF70",
#'   "#EE6363", "#00CD00", "#436EEE", "#FFAEB9",
#'   "#FFA54F", "#68228B", "#636363"
#' )
#'
#' # onestep sankey
#' sankeyLand(
#'   dataset = SL_2002_2014$lulc_Onestep,
#'   legendtable = SL_2002_2014$tb_legend
#' )
#'
#' # multistep sankey
#' sankeyLand(
#'   dataset = SL_2002_2014$lulc_Multistep,
#'   legendtable = SL_2002_2014$tb_legend
#' )
#'
sankeyLand <- function(dataset, legendtable, iterations = 0) {
  linkMultistep <- dataset %>%
    left_join(legendtable, by = c("From" = "categoryValue")) %>%
    left_join(legendtable, by = c("To" = "categoryValue")) %>%
    dplyr::select(-c(From, To)) %>%
    rename(
      "From" = "categoryName.x",
      "To" = "categoryName.y",
      "colorFrom" = "color.x",
      "colorTo" = "color.y"
    ) %>%
    tidyr::unite("source",
      c("From", "yearFrom"),
      sep = "-",
      remove = FALSE
    ) %>%
    tidyr::unite("target",
      c("To", "yearTo"),
      sep = "-",
      remove = FALSE
    ) %>%
    dplyr::select(source, target, From, To, km2, yearFrom, yearTo)
  # defining the color scale
  domain <- paste(
    paste0(
      "'",
      as.character(levels(legendtable$categoryName)), "'"
    ),
    collapse = ", "
  )

  range <- paste(
    paste0(
      "'",
      as.character(legendtable$color[order(legendtable$categoryName)]), "'"
    ),
    collapse = ", "
  )

  colorScale <-
    paste0(
      "d3.scaleOrdinal().domain([",
      domain,
      "]).range([",
      range,
      "]).unknown(['grey']);"
    )


  nodeMultistep <-
    data.frame(
      name = unique(c(
        as.character(linkMultistep[order(linkMultistep$From), ]$source),
        as.character(linkMultistep[order(linkMultistep$To), ]$target)
      ))
    ) %>%
    tidyr::separate(name, c("name02", "year"), sep = "-", remove = FALSE)

  linkMultistep$IDsource <-
    match(linkMultistep$source, nodeMultistep$name) - 1
  linkMultistep$IDtarget <-
    match(linkMultistep$target, nodeMultistep$name) - 1

  # Plot
  networkD3::sankeyNetwork(
    Links = as.data.frame(linkMultistep),
    Nodes = nodeMultistep,
    Source = "IDsource",
    Target = "IDtarget",
    colourScale = colorScale,
    Value = "km2",
    NodeID = "name02",
    fontSize = 13,
    nodeWidth = 20,
    fontFamily = "sans-serif",
    iterations = iterations,
    nodePadding = 20,
    sinksRight = FALSE
  )
}

context("The plots")

set.seed(159)
demo_raster <- .demo_landscape(
  year = 2000:2005,
  res = 1,
  crs = "+proj=utm +zone=21 +south +ellps=GRS80 +units=m +no_defs"
)
demo_cont <- contingencyTable(demo_raster, pixelresolution = 1)

demo_int_km2 <-
  intensityAnalysis(
    dataset = demo_cont,
    category_n = "GUP",
    category_m = "SXQ",
    area_km2 = TRUE
  )


test_that("Behavior of the plot methods", {
  expect_invisible(OpenLand::plot(demo_int_km2$interval_lvl))
  expect_invisible(OpenLand::plot(demo_int_km2$interval_lvl, title = "Main Title"))
  expect_output(str(OpenLand::plot(demo_int_km2$interval_lvl)), "gtable, containing")


  expect_invisible(OpenLand::plot(demo_int_km2$category_lvlGain))
  expect_invisible(OpenLand::plot(demo_int_km2$category_lvlGain, title = "Main Title"))
  expect_output(str(OpenLand::plot(demo_int_km2$category_lvlGain)), "gtable, containing")


  expect_invisible(OpenLand::plot(demo_int_km2$transition_lvlGain_n))
  expect_invisible(OpenLand::plot(demo_int_km2$transition_lvlGain_n, title = "Main Title"))
  expect_output(str(OpenLand::plot(demo_int_km2$transition_lvlGain_n)), "gtable, containing")
})


test_that("Behavior of the other plots", {
  expect_visible(barplotLand(demo_cont$lulc_Multistep, demo_cont$tb_legend, area_km2 = TRUE))
  expect_invisible(chordDiagramLand(demo_cont$lulc_Onestep, demo_cont$tb_legend, area_km2 = TRUE))
  expect_visible(netgrossplot(demo_cont$lulc_Multistep, demo_cont$tb_legend, area_km2 = TRUE))

  expect_visible(barplotLand(demo_cont$lulc_Multistep, demo_cont$tb_legend, area_km2 = FALSE))
  expect_invisible(chordDiagramLand(demo_cont$lulc_Onestep, demo_cont$tb_legend, area_km2 = FALSE))
  expect_visible(netgrossplot(demo_cont$lulc_Multistep, demo_cont$tb_legend, area_km2 = FALSE))

  expect_visible(sankeyLand(demo_cont$lulc_Onestep, demo_cont$tb_legend))
  expect_visible(sankeyLand(demo_cont$lulc_Multistep, demo_cont$tb_legend))


  expect_s3_class(barplotLand(demo_cont$lulc_Multistep, demo_cont$tb_legend), "ggplot")
  expect_s3_class(netgrossplot(demo_cont$lulc_Multistep, demo_cont$tb_legend), "ggplot")
  expect_output(str(sankeyLand(demo_cont$lulc_Onestep, demo_cont$tb_legend)), "List of 8")
  expect_output(str(sankeyLand(demo_cont$lulc_Multistep, demo_cont$tb_legend)), "List of 8")
})

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/intensityClass.R
\docType{class}
\name{Category-class}
\alias{Category-class}
\alias{Category}
\title{Class Category}
\description{
A S4 class for the Category level result of an Intensity analysis.  Can be
plotted with the plot method \code{\link{plot}}.
}
\details{
The slots \code{categoryData} and \code{categoryStationarity} can receive
   tables for "Gain" or "Loss" in the following format:
\enumerate{

\item Gain

 \itemize{

 \item \code{categoryData}: \code{<tibble>}. A table containing 6 columns:
 \enumerate{
   \item Period: \code{<fct>}. The period \emph{[Yt, Yt+1]}.
   \item To: \code{<fct>}. A LUC category \emph{j}.
   \item Interval: \code{<int>}. Duration of the period \emph{[Yt, Yt+1]} in years.
   \item GG_km2/GG_pixel: \code{<dbl>/<int>}. Area of gross gain of category
   \emph{j} during \emph{[Yt, Yt+1]}.
   \item Gtj: \code{<dbl>}. Annual intensity of gross gain of category \emph{j}
   for time interval \emph{[Yt, Yt+1]}.
   \item St: \code{<dbl>}. Annual intensity of change for time interval \emph{[Yt, Yt+1]}.
   }
 \item categoryStationarity:  \code{<tibble>}. A table with the results of a
 stationarity test of the gain of the categories on the Category level, containing 5 columns:
 \enumerate{
   \item To: \code{<fct>}. A category of interest \emph{j}.
   \item gain: \code{<int>}. Number of times a category had gains during all
   time intervals \emph{[Y1, YT]}.
   \item N: \code{<int>}. Total number of evaluated time points (T).
   \item Stationarity: \code{<chr>}. \emph{Active Gain} or \emph{Dormant Gain}.
   \item Test: \code{<chr>}. \emph{Y} if stationarity was detected and \emph{N} if not.
   }
   }

\item Loss

 \itemize{

 \item \code{categoryData}: \code{<tibble>}. A table containing 6 columns:
 \enumerate{
   \item Period: \code{<fct>}. The period \emph{[Yt, Yt+1]}.
   \item From: \code{<fct>}. A LUC category \emph{i}.
   \item Interval: \code{<int>}. Duration of the period \emph{[Yt, Yt+1]} in years.
   \item GG_km2/GG_pixel: \code{<dbl>/<int>}. Area of gross loss of category
   \emph{i} during \emph{[Yt, Yt+1]}.
   \item Lti: \code{<dbl>}. Annual intensity of gross loss of category \emph{i}
   for time interval \emph{[Yt, Yt+1]}.
   \item STt: \code{<dbl>}. Annual intensity of change for time interval \emph{[Yt, Yt+1]}.
   }
  \item categoryStationarity: \code{<tibble>}. A table of stationarity test
  over the loss of the categories in the Category level, containing 5 columns:
 \enumerate{
   \item From: \code{<fct>}. A category of interest \emph{i}.
   \item loss: \code{<int>}. Number of times a category had losses during all
   time intervals \emph{[Y1, YT]}.
   \item N: \code{<int>}. Total number of evaluated time points (T).
   \item Stationarity: \code{<chr>}. \emph{Active Loss} or \emph{Dormant Loss}.
   \item Test: \code{<chr>}. \emph{Y} if stationarity was detected and \emph{N} if not.
   }
   }

   }
}
\section{Slots}{

\describe{
\item{\code{lookupcolor}}{The colors (character vector) associated with the LUC legend items.}

\item{\code{categoryData}}{tibble. A table of Category level's results (gain (\emph{Gtj})
or loss (\emph{Lti}) values).}

\item{\code{categoryStationarity}}{tibble. A table containing results of a stationarity
test. A change is considered stationary only if the intensities for all time
intervals reside on one side of the uniform intensity, i.e are smaller or bigger
than the uniform rate over the whole period.}
}}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/intensityAnalysis.R
\name{intensityAnalysis}
\alias{intensityAnalysis}
\title{Performs the intensity analysis based on cross-tabulation matrices of each
time step}
\usage{
intensityAnalysis(dataset, category_n, category_m, area_km2 = TRUE)
}
\arguments{
\item{dataset}{list. The result object from \code{\link{contingencyTable}}.}

\item{category_n}{character. The gaining category in the transition of interest (\emph{n}).}

\item{category_m}{character. The losing category in the transition of interest (\emph{m}).}

\item{area_km2}{logical. If TRUE the change is computed in km2, if FALSE in pixel counts.}
}
\value{
Intensity object
}
\description{
This function implements an Intensity Analysis (IA) according to Aldwaik &
Pontius (2012), a quantitative method to analyze time series of land use and
cover (LUC) maps.  For IA, a cross-tabulation matrix is composed for each LUC
transition step in time.
}
\details{
IA includes three levels of analysis of LUC changes. Consecutive analysis
levels detail hereby information given by the previous analysis level
\cite{(Aldwaik and Pontius, 2012, 2013)}.


\enumerate{
 \item The \emph{interval level} examines how the size and speed of change
 vary across time intervals.
 \item The \emph{category level} examines how the size and intensity of gross
 losses and gross gains in each category vary across categories for each time
 interval.
 \item The \emph{transition level} examines how the size and intensity of a
 category’s transitions vary across the other categories that are available
 for that transition.
  }


At each analysis level, the method tests for stationarity of patterns across
time intervals.

\bold{The function returns a list with 6 objects:}
\enumerate{
 \item lulc_table: \code{tibble}. Contingency table of LUC transitions at all
  analysed time steps, containing 6 columns:
   \enumerate{
   \item Period:  \code{<fct>}. Evaluated period of transition in the format
   \code{year t - year t+1}.
   \item From: \code{<fct>}. The category in year t.
   \item To: \code{<fct>}. The category in year t+1.
   \item km2: \code{<dbl>}. Area in square kilometers that transited from the
    category \code{From}.
   to the category \code{To} in the period.
   \item QtPixel: \code{<int>}. Number of pixels that transited from.
   the category \code{From} to the category \code{To} in the period.
   \item Interval: \code{<int>}. Interval in years of the evaluated period.

   }

 \item \emph{lv1_tbl}: An \code{\linkS4class{Interval}} object containing the
  \emph{St} and \emph{U} values.
 \item \emph{category_lvlGain}: A \code{\linkS4class{Category}} object
 containing the gain of the LUC category in a period (\emph{Gtj}).
 \item \emph{category_lvlLoss}: A \code{\linkS4class{Category}} object
 containing the loss of the LUC category in a period (\emph{Lti}).
 \item \emph{transition_lvlGain_n}: A \code{\linkS4class{Transition}} object
 containing the annualized rate of gain in \emph{category n} (\emph{Rtin}) and
 the respective Uniform Intensity (\emph{Wtn}).
 \item \emph{transition_lvlLoss_m}: A \code{\linkS4class{Transition}} object
 containing the annualized rate of loss in \emph{category m} (\emph{Qtmj}) and
 the respective Uniform Intensity (\emph{Vtm}).

  }
}
\examples{

# editing the category name

SL_2002_2014$tb_legend$categoryName <- factor(
  c(
    "Ap", "FF", "SA", "SG", "aa", "SF",
    "Agua", "Iu", "Ac", "R", "Im"
  ),
  levels = c(
    "FF", "SF", "SA", "SG", "aa", "Ap",
    "Ac", "Im", "Iu", "Agua", "R"
  )
)

SL_2002_2014$tb_legend$color <- c(
  "#FFE4B5", "#228B22", "#00FF00", "#CAFF70",
  "#EE6363", "#00CD00", "#436EEE", "#FFAEB9",
  "#FFA54F", "#68228B", "#636363"
)

intensityAnalysis(dataset = SL_2002_2014, category_n = "Ap", category_m = "SG", area_km2 = TRUE)

}
\references{
Aldwaik, S. Z. and Pontius, R. G. (2012) ‘Intensity analysis to unify
measurements of size and stationarity of land changes by interval, category, and
transition, Landscape and Urban Planning. Elsevier B.V., 106(1), pp. 103–114.
\doi{10.1016/j.landurbplan.2012.02.010}.

Aldwaik, S. Z. and Pontius, R. G. (2013) ‘Map errors that could account for deviations
from a uniform intensity of land change, International Journal of Geographical
Information Science. Taylor & Francis, 27(9), pp. 1717–1739. \doi{10.1080/13658816.2013.787618}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/generalfunctions.R
\name{acc_changes}
\alias{acc_changes}
\title{Accumulates changes in a LULC raster time series}
\usage{
acc_changes(path)
}
\arguments{
\item{path}{The path for the Raster* directory or list of Raster* to be
analysed.}
}
\value{
Two objects, a RasterLayer and a table.
}
\description{
This function calculates the number of times a pixel has changed during
the analysed period. It returns a raster with the number of changes as
pixel value and a table containing the areal percentage of every pixel value
(number of changes).
}
\examples{
\donttest{
url <- "https://zenodo.org/record/3685230/files/SaoLourencoBasin.rda?download=1"
if (OpenLand:::.openland_try_download_and_load_rda(url,
  object = "SaoLourencoBasin", timeout = 10,
  cache = FALSE
)) {
  acc_changes(SaoLourencoBasin)
}
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/otherplots.R
\name{chordDiagramLand}
\alias{chordDiagramLand}
\title{One step transitions (Chord diagram)}
\usage{
chordDiagramLand(
  dataset,
  legendtable,
  legposition = c(x = -1.3, y = 0),
  legtitle = "Categories",
  sectorcol = "gray80",
  area_km2 = TRUE,
  legendsize = 1,
  y.intersp = 1,
  x.margin = c(-1, 1)
)
}
\arguments{
\item{dataset}{A table of the one step transition (\code{lulc_OneStep}) generated
by \code{\link{contingencyTable}}.}

\item{legendtable}{A table containing the LUC legend items and their respective
color (\code{tb_legend}).}

\item{legposition}{numeric. A vector containing the `x` and `y` values for the
position of the legend. (see \code{\link[graphics]{legend}}).}

\item{legtitle}{character. The title of the legend.}

\item{sectorcol}{character. The color of the external sector containing the years
of compared time points.}

\item{area_km2}{logical. If TRUE the change is computed in km2, if FALSE in
pixel counts.}

\item{legendsize}{numeric. Font size of the legend. (see "cex" in \code{\link[graphics]{legend}}).}

\item{y.intersp}{numeric. character interspacing factor for vertical (y)
spacing in the legend.}

\item{x.margin}{numeric vector ensuring additional space (blank area) on the
left or right of the circle for the legend, by default it is c(-1, 1). (see
"canvas.xlim" in \code{\link[circlize]{circos.par}})}
}
\value{
A Chord Diagram
}
\description{
A circlize plot representing the one step transitions between two times point
of interest.
}
\examples{

# editing the category names

SL_2002_2014$tb_legend$categoryName <- factor(
  c(
    "Ap", "FF", "SA", "SG", "aa", "SF",
    "Agua", "Iu", "Ac", "R", "Im"
  ),
  levels = c(
    "FF", "SF", "SA", "SG", "aa", "Ap",
    "Ac", "Im", "Iu", "Agua", "R"
  )
)

SL_2002_2014$tb_legend$color <- c(
  "#FFE4B5", "#228B22", "#00FF00", "#CAFF70",
  "#EE6363", "#00CD00", "#436EEE", "#FFAEB9",
  "#FFA54F", "#68228B", "#636363"
)

# the plot
chordDiagramLand(
  dataset = SL_2002_2014$lulc_Onestep,
  legendtable = SL_2002_2014$tb_legend
)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotMethods.R
\docType{methods}
\name{plot}
\alias{plot}
\alias{plot,ANY,ANY-method}
\alias{plot,Interval,ANY-method}
\alias{plot,Category,ANY-method}
\alias{plot,Transition,ANY-method}
\alias{plot,Interval-method}
\alias{plot,Category-method}
\alias{plot,Transition-method}
\title{Methods for function \code{plot} in package \pkg{OpenLand}}
\usage{
plot(x, y, ...)

\S4method{plot}{Interval}(
  x,
  y,
  labels = c(leftlabel = "Interval Change Area (percent of map)", rightlabel =
    "Annual Change Area (percent of map)"),
  title = NA,
  labs = c(type = "Changes", ur = "Uniform Intensity"),
  marginplot = c(lh = -10, rh = 0),
  leg_curv = c(x = 0.1, y = 0.1),
  color_bar = c(fast = "#B22222", slow = "#006400", area = "gray40"),
  fontsize_ui = 10,
  ...
)

\S4method{plot}{Category}(
  x,
  y,
  labels = c(leftlabel = "Annual Change Area (km2 or pixels)", rightlabel =
    "Annual Change Intensity (percent of category)"),
  title = NA,
  labs = c(type = "Categories", ur = "Uniform Intensity"),
  marginplot = c(lh = 0.5, rh = 0.5),
  leg_curv = c(x = 0.1, y = 0.1),
  fontsize_ui = 10,
  ...
)

\S4method{plot}{Transition}(
  x,
  y,
  labels = c(leftlabel = "Annual Transition Area (km2 or pixels)", rightlabel =
    "Annual Transition Intensity (percent of category)"),
  title = NA,
  labs = c(type = "Categories", ur = "Uniform Intensity"),
  marginplot = c(lh = 0.5, rh = 0.5),
  leg_curv = c(x = 0.1, y = 0.1),
  fontsize_ui = 10,
  ...
)
}
\arguments{
\item{x}{An intensity object generated by \code{\link{intensityAnalysis}}.}

\item{y}{ignored.}

\item{\dots}{additional arguments for theme parameters from ggplot2, see
\code{\link[ggplot2]{theme}}.}

\item{labels}{character. Left and right axis titles(caption).}

\item{title}{character. Main title.}

\item{labs}{character. The lateral legend.}

\item{marginplot}{numeric. Adjustment of the origins of left and right part of
the plots.}

\item{leg_curv}{numeric. x and y values that control the arrow size and position
pointing to the Uniform Intensity vertical line.}

\item{color_bar}{character. Colors defined for the fast, slow and area bars
(only for an \code{\linkS4class{Interval}} object).}

\item{fontsize_ui}{numeric. Fontsize of the uniform intensity percent in the plot.}

\item{Interval}{The class.}

\item{Category}{The class.}

\item{Transition}{The class.}
}
\value{
An intensity graph
}
\description{
Plot \code{Intensity} objects based on Intensity Analysis output.
}
\keyword{methods}
\keyword{plot}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/otherplots.R
\name{barplotLand}
\alias{barplotLand}
\title{Area of LUC categories at time points}
\usage{
barplotLand(
  dataset,
  legendtable,
  title = NULL,
  caption = "LUC Categories",
  xlab = "Year",
  ylab = "Area (km2 or pixel)",
  area_km2 = TRUE,
  ...
)
}
\arguments{
\item{dataset}{A table of the multi step transitions (\code{lulc_Multistep})
generated by \code{\link{contingencyTable}}.}

\item{legendtable}{A table containing the LUC legend items and their respective
color (\code{tb_legend}).}

\item{title}{character. The title of the plot.}

\item{caption}{character. The caption of the plot.}

\item{xlab}{character. Label for the x axis.}

\item{ylab}{character. Label for the y axis.}

\item{area_km2}{logical. If TRUE the change is computed in km2, if FALSE in
pixel counts.}

\item{\dots}{additional themes parameters, see \code{\link[ggplot2]{theme}}.}
}
\value{
a barplot
}
\description{
A grouped barplot representing the areas of LUC categories at each time point
of the analysed period.
}
\examples{

# editing the category names

SL_2002_2014$tb_legend$categoryName <- factor(
  c(
    "Ap", "FF", "SA", "SG", "aa", "SF",
    "Agua", "Iu", "Ac", "R", "Im"
  ),
  levels = c(
    "FF", "SF", "SA", "SG", "aa", "Ap",
    "Ac", "Im", "Iu", "Agua", "R"
  )
)

SL_2002_2014$tb_legend$color <- c(
  "#FFE4B5", "#228B22", "#00FF00", "#CAFF70",
  "#EE6363", "#00CD00", "#436EEE", "#FFAEB9",
  "#FFA54F", "#68228B", "#636363"
)
# the plot
barplotLand(
  dataset = SL_2002_2014$lulc_Multistep,
  legendtable = SL_2002_2014$tb_legend,
  area_km2 = TRUE
)

}
\seealso{
\code{ggplot2::\link[ggplot2]{theme}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/otherplots.R
\name{netgrossplot}
\alias{netgrossplot}
\title{Net and gross changes of LUC categories}
\usage{
netgrossplot(
  dataset,
  legendtable,
  title = NULL,
  xlab = "LUC category",
  ylab = "Area (Km2)",
  legend_title = "Changes",
  changesLabel = c(GC = "Gross change", NG = "Net gain", NL = "Net loss"),
  color = c(GC = "gray70", NG = "#006400", NL = "#EE2C2C"),
  area_km2 = TRUE
)
}
\arguments{
\item{dataset}{A table of the multi step transition (\code{lulc_Mulstistep})
generated by \code{\link{contingencyTable}}.}

\item{legendtable}{A table containing the LUC legend items and their respective
color (\code{tb_legend}).}

\item{title}{character. The title of the plot (optional), use \code{NULL} for
no title.}

\item{xlab}{character. Label for the x axis.}

\item{ylab}{character. Label for the y axis.}

\item{legend_title}{character. The title of the legend.}

\item{changesLabel}{character. Labels for the three types of changes, defaults
are c(GC = "Gross change", NG = "Net gain", NL = "Net loss").}

\item{color}{character. A vector defining the three bar colors.}

\item{area_km2}{logical. If TRUE the change is computed in km2, if FALSE in
pixel counts.}
}
\value{
A bar plot
}
\description{
A stacked barplot showing net and gross changes of LUC categories during the
entire analysed time period.
}
\examples{

# editing the category names

SL_2002_2014$tb_legend$categoryName <- factor(
  c(
    "Ap", "FF", "SA", "SG", "aa", "SF",
    "Agua", "Iu", "Ac", "R", "Im"
  ),
  levels = c(
    "FF", "SF", "SA", "SG", "aa", "Ap",
    "Ac", "Im", "Iu", "Agua", "R"
  )
)

# the plot
netgrossplot(
  dataset = SL_2002_2014$lulc_Multistep,
  legendtable = SL_2002_2014$tb_legend,
  title = NULL,
  xlab = "LUC Category",
  changes = c(GC = "Gross changes", NG = "Net Gain", NL = "Net Loss"),
  color = c(GC = "gray70", NG = "#006400", NL = "#EE2C2C")
)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/generalfunctions.R
\name{summary_dir}
\alias{summary_dir}
\title{Summary of multiple parameters in a raster directory}
\usage{
summary_dir(path)
}
\arguments{
\item{path}{The path for the Raster* directory or list of Raster* to be
analysed.}
}
\value{
Table with the raster parameters in columns
}
\description{
Listing major characteristics of raster inputs. Those characteristics are the
dimensions, the resolution, the extent, the values (min, max) and the
coordinate reference system.
}
\examples{
\donttest{
url <- "https://zenodo.org/record/3685230/files/SaoLourencoBasin.rda?download=1"
if (OpenLand:::.openland_try_download_and_load_rda(url,
  object = "SaoLourencoBasin", timeout = 10,
  cache = FALSE
)) {
  summary_dir(raster::unstack(SaoLourencoBasin))
}
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/generalfunctions.R
\name{summary_map}
\alias{summary_map}
\title{Quantitative summary of a unique categorical raster}
\usage{
summary_map(path)
}
\arguments{
\item{path}{The path for the raster to be analysed, if path is a multilayer
raster only the first RasterLayer will be analysed.}
}
\value{
A table containing in columns the pixel counts for each pixel value
}
\description{
This function presents a summary with the pixel quantity of each category
present in a categorical raster.
}
\examples{
\donttest{
url <- "https://zenodo.org/record/3685230/files/SaoLourencoBasin.rda?download=1"
if (OpenLand:::.openland_try_download_and_load_rda(url,
  object = "SaoLourencoBasin", timeout = 10,
  cache = FALSE
)) {
  summary_map(SaoLourencoBasin[[1]])
}
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/contingencyTable.R
\name{contingencyTable}
\alias{contingencyTable}
\title{Contingency table}
\usage{
contingencyTable(input_raster, pixelresolution = 30)
}
\arguments{
\item{input_raster}{path (character), Raster* object or list of Raster*
objects. See \cr \code{\link[raster]{raster}} for more information about
supported file types.}

\item{pixelresolution}{numeric. The pixel spatial resolution in meter.}
}
\value{
A list that contains 5 objects.
\itemize{
  \item \code{lulc_Mulstistep}: \code{<tibble>} Contingency table for all
  analysed time steps, containing 8 columns:
  \enumerate{
  \item Period: \code{<chr>} The period \emph{[Yt, Yt+1]}.
  \item From: \code{<dbl>} numerical code of a LUC category \emph{i}.
  \item To: \code{<dbl>} numerical code of a LUC category \emph{j}.
  \item km2: \code{<dbl>} Area in square kilometers that transited from the
  category \emph{i}
   to category \emph{j} in the period from \emph{Yt} to \emph{Yt+1}.
  \item Interval: \code{<dbl>} Interval of years between the first and
   the last year of the period \emph{[Yt, Yt+1]}.
  \item QtPixel: \code{<int>} Pixel count that transited from the categories
   \emph{i}
   to category \emph{j} in the period from \emph{Yt} to \emph{Yt+1}.
  \item yearFrom: \code{<chr>} The year that the change comes from \emph{[Yt]}.
  \item yearTo: \code{<chr>} The year that the change goes for \emph{[Yt+1]}.
  }
  \item \code{lulc_Onestep}:\code{<tibble>} Contingency table for the entire
  analysed period \emph{[Y1, YT]}, containing
  8 columns identical with \code{lulc_Mulstistep}.
  \item \code{tb_legend}: \code{<tibble>} A table of the pixel value, his
  name and color containing 3 columns:
  \enumerate{
  \item categoryValue: \code{<dbl>} the pixel value of the LUC category.
  \item categoryName: \code{<factor>} randomly created string associated with
   a given pixel value of a LUC category.
  \item color: \code{<chr>} random color associated with the given pixel value
   of a LUC category.
  Before further analysis, one would like to change the \code{categoryName}
  and \code{color} values.
    \itemize{
        \item Therefore the category names have to be in the same order as the
         \code{categoryValue}
        and the \code{levels} should be put in the right order for legend
        plotting. Like:
        \preformatted{

        myobject$tb_legend$categoryName <- factor(c("name1", "name2", "name3", "name4"),
                                               levels = c("name3", "name2", "name1", "name4"))}
        \item The colors have to in the same order as the values in the
         \code{categoryValue} column. Colors can be given by the color name
         (eg. "black") or an HEX value (eg. #FFFFFF). Like:
       \preformatted{

       myobject$tb_legend$color <- c("#CDB79E", "red", "#66CD00", "yellow")}}}
  \item \code{totalArea}: \code{<tibble>}  A table with the total area of the
   study area containing 2 columns:
  \enumerate{
  \item area_km2: \code{<numeric>} The total area in square kilometers.
  \item QtPixel: \code{<numeric>} The total area in pixel counts.
  }
  \item \code{totalInterval}: \code{<numeric>} Total interval of the analysed
   time series in years.
  }
}
\description{
Extracts LUC transitions for all input grids of the time series.
}
\examples{
\donttest{
url <- "https://zenodo.org/record/3685230/files/SaoLourencoBasin.rda?download=1"
if (OpenLand:::.openland_try_download_and_load_rda(url,
  object = "SaoLourencoBasin", timeout = 10,
  cache = FALSE
)) {
  # the contingencyTable() function, with the SaoLourencoBasin dataset
  contingencyTable(input_raster = SaoLourencoBasin, pixelresolution = 30)
}
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/otherplots.R
\name{sankeyLand}
\alias{sankeyLand}
\title{Sankey diagram of LUC transitions (one or multistep)}
\usage{
sankeyLand(dataset, legendtable, iterations = 0)
}
\arguments{
\item{dataset}{A table of the multi step (\code{lulc_Mulstistep}).
or one step transitions (\code{lulc_OneStep}) generated by \code{\link{contingencyTable}}.}

\item{legendtable}{A table containing the LUC legend items and their respective
color (\code{tb_legend}).}

\item{iterations}{numeric. Number of iterations in the diagram layout for
computation of the depth (y-position) of each node. See \code{\link[networkD3]{sankeyNetwork}}.}
}
\value{
A sankey diagram
}
\description{
A sankey showing the one or multi step LUC transitions during the analysed period.
}
\examples{

# editing the category names

SL_2002_2014$tb_legend$categoryName <- factor(
  c(
    "Ap", "FF", "SA", "SG", "aa", "SF",
    "Agua", "Iu", "Ac", "R", "Im"
  ),
  levels = c(
    "FF", "SF", "SA", "SG", "aa", "Ap",
    "Ac", "Im", "Iu", "Agua", "R"
  )
)

SL_2002_2014$tb_legend$color <- c(
  "#FFE4B5", "#228B22", "#00FF00", "#CAFF70",
  "#EE6363", "#00CD00", "#436EEE", "#FFAEB9",
  "#FFA54F", "#68228B", "#636363"
)

# onestep sankey
sankeyLand(
  dataset = SL_2002_2014$lulc_Onestep,
  legendtable = SL_2002_2014$tb_legend
)

# multistep sankey
sankeyLand(
  dataset = SL_2002_2014$lulc_Multistep,
  legendtable = SL_2002_2014$tb_legend
)

}
\seealso{
\code{\link[networkD3]{sankeyNetwork}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/generic_method.R
\name{$,Interval-method}
\alias{$,Interval-method}
\alias{$,Category-method}
\alias{$,Transition-method}
\title{Accessor method for objects from Intensity Analysis}
\usage{
\S4method{$}{Interval}(x, name)

\S4method{$}{Category}(x, name)

\S4method{$}{Transition}(x, name)
}
\arguments{
\item{x}{the object}

\item{name}{the name of the object}

\item{Interval}{the class}

\item{Category}{the class}

\item{Transition}{the class}
}
\description{
Accessor method for objects from Intensity Analysis
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/demolandscape.R
\name{.demo_landscape}
\alias{.demo_landscape}
\title{Create Raster with Random pixel Value}
\usage{
.demo_landscape(
  year,
  nrows = 100,
  ncols = 100,
  res = 1,
  xmn = 0,
  xmx = 100,
  ymn = 0,
  ymx = 100,
  crs = NA,
  category = 1:5,
  prob = NULL
)
}
\arguments{
\item{year}{numeric. A vector of year, first and last included.}

\item{nrows}{numeric. nrows of the raster to be created.}

\item{ncols}{numeric. ncols of the raster to be created.}

\item{res}{numeric. the resolution of the raster to be created.}

\item{xmn}{numeric. x minimum extent.}

\item{xmx}{numeric. x maximum extent.}

\item{ymn}{numeric. y minimum extent.}

\item{ymx}{numeric. y maximum extent.}

\item{crs}{character. the coordinate referencing system.}

\item{category}{A numeric vector of the raster categories.}

\item{prob}{A numeric vector of the probability of occurrence for the
category list.}
}
\value{
list
}
\description{
This function creates a raster series with some setup like the layer name and
the sample value for the lulc
}
\examples{
.demo_landscape(
  year = 2000:2005,
  res = 1,
  crs = "+proj=utm +zone=21 +south +ellps=GRS80 +units=m +no_defs"
)

}
\seealso{
\code{\link[raster]{raster}}
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rasters_input.R
\name{.input_rasters}
\alias{.input_rasters}
\alias{.input_rasters,character-method}
\alias{character}
\alias{.input_rasters,list-method}
\alias{list}
\alias{.input_rasters,RasterLayer-method}
\alias{RasterLayer}
\alias{.input_rasters,RasterBrick-method}
\alias{RasterBrick}
\alias{.input_rasters,RasterStack-method}
\alias{RasterStack}
\title{input_rasters}
\usage{
.input_rasters(x, ...)

\S4method{.input_rasters}{character}(x, ...)

\S4method{.input_rasters}{list}(x, ...)

\S4method{.input_rasters}{RasterLayer}(x, ...)

\S4method{.input_rasters}{RasterBrick}(x, ...)

\S4method{.input_rasters}{RasterStack}(x, ...)
}
\arguments{
\item{x}{path (character), Raster* object or list of Raster* objects.}

\item{\dots}{additional arguments to \code{raster::\link[raster]{raster}}.}
}
\value{
A RasterStack
}
\description{
A methods for loading the raster into OpenLand
}
\seealso{
\code{raster::\link[raster]{raster}},
\code{raster::\link[raster]{stack}}
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/intensityClass.R
\docType{class}
\name{Interval-class}
\alias{Interval-class}
\alias{Interval}
\title{Class Interval}
\description{
A S4 class for the Interval level result of an Intensity analysis.  Can be
plotted with the plot method \code{\link{plot}}.
}
\details{
The slot \code{intervalData} receives a table containing 4 columns
in the following format:
   \enumerate{
   \item Period: \code{<fct>}. The period of interest \emph{[Yt, Yt+1]}.
   \item PercentChange: \code{<dbl>}. Changed area on the Interval level (\%).
   \item St: \code{<dbl>}. Annual intensity of change for a time
   period [Yt, Yt+1].
   \item U: \code{<dbl>}. Uniform intensity for a LUC category change in a time
   period of interest.
   }
}
\section{Slots}{

\describe{
\item{\code{intervalData}}{tibble. A table with the results of an Intensity analysis
at the Interval level (\emph{St} and \emph{U} values).}
}}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/generalfunctions.R
\name{.openland_try_download_and_load_rda}
\alias{.openland_try_download_and_load_rda}
\title{Download and load a remote .rda dataset (graceful)}
\usage{
.openland_try_download_and_load_rda(
  url,
  object = NULL,
  envir = parent.frame(),
  timeout = 10,
  cache = TRUE,
  cache_dir = NULL,
  quiet = TRUE
)
}
\arguments{
\item{url}{character. Remote `.rda` URL.}

\item{object}{character or NULL. If provided, requires that this object name
exists inside the `.rda` file; otherwise returns `FALSE`.}

\item{envir}{environment. Environment where the objects from the `.rda` file
will be loaded.}

\item{timeout}{integer. Download timeout (seconds) set via `options(timeout)`.}

\item{cache}{logical. Whether to cache the downloaded file.}

\item{cache_dir}{character or NULL. Cache directory used only when
`cache = TRUE`. When `NULL`, uses `tools::R_user_dir("OpenLand",
which = "cache")` when available.}

\item{quiet}{logical. Passed to `utils::download.file()`.}
}
\value{
logical. `TRUE` if the `.rda` was successfully downloaded (or found
  in cache) and loaded into `envir`; `FALSE` otherwise.
}
\description{
Internal helper to download a remote `.rda` file (e.g., from Zenodo), load it
into an environment, and return a single `TRUE/FALSE` indicating whether the
dataset is available.
}
\details{
The function is defensive by design: it should not error if the remote host is
unavailable, if the download fails, or if the `.rda` cannot be loaded. It can
optionally cache the downloaded file in a persistent user cache directory. When
`cache = FALSE`, the file is downloaded to a temporary path and removed at the
end of the call.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/intensityClass.R
\docType{class}
\name{Transition-class}
\alias{Transition-class}
\alias{Transition}
\title{Class Transition}
\description{
A S4 class for the Transition level result of an Intensity analysis. Can be
plotted with the plot method \code{\link{plot}}.
}
\details{
The slots \code{transitionData} and \code{transitionStationarity} can
receive tables for "Gain of category n" or "Loss of category m" in the following
format:
\enumerate{

\item Gain of category n:

 \itemize{

  \item \code{transitionData}: \code{<tibble>}. A table with 7 columns:
  \enumerate{
   \item Period: \code{<fct>}. The period \emph{[Yt, Yt+1]}.
   \item From: \code{<fct>}. A category i.
   \item To: \code{<fct>}. The gaining category in the transition of interest \emph{(n)}.
   \item Interval: \code{<int>}. Duration of the period \emph{[Yt, Yt+1]}.
   \item T_i2n_km2/T_i2n_pixel: \code{<dbl>}. Area with transition from category
   i to category n during time interval \emph{[Yt, Yt+1]} where \emph{i}
   \code{is not equal to} \emph{n}.
   \item Rtin: \code{<dbl>}. Annual intensity of transition from category i to
   category n
   during time interval \emph{[Yt, Yt+1]} where \emph{i}  \code{is not equal to} \emph{n}.
   \item Wtn: \code{<dbl>}. Value of the uniform intensity of the transition
   to category n from all non-n categories at time Yt during time interval \emph{[Yt, Yt+1]}.
   }
 \item transitionStationarity: \code{<tibble>}. A table containing results of
 a stationarity test over the gain on \emph{category n} containing 5 columns:
 \enumerate{
   \item From: \code{<fct>}. The losing category in the transition of interest
   to the category n.
   \item loss: \code{<int>}. Number of times the category had losses to the
   category n.
   \item N: \code{<int>}. Total number of transitions to be considered as stationary (T).
   \item Stationarity: \code{<chr>}. \emph{targeted by} or \emph{avoided by}
   the category \code{n}.
   \item Test: \code{<chr>}. \emph{Y} for stationarity detected and \emph{N}
   when not.
   }
   }

\item Loss of category m:

 \itemize{

  \item \code{transitionData}: \code{<tibble>}. A table with 7 columns:
  \enumerate{
   \item Period: \code{<fct>}. The period \emph{[Yt, Yt+1]}.
   \item To: \code{<fct>}. A category \emph{j}.
   \item From: \code{<fct>}. The losing category in the transition of interest (m).
   \item Interval: \code{<dbl>}. Duration of the period \emph{[Yt, Yt+1]}.
   \item T_m2j_km2/T_m2j_pixel: \code{<dbl>}. Area with transition from category
    \emph{m} to category \emph{j} during time interval \emph{[Yt, Yt+1]}
   where \emph{j}  \code{is not equal to} \emph{m}.
   \item Qtmj: \code{<dbl>}. Annual intensity of transition from category \emph{m} to
   category \emph{j} during time interval \emph{[Yt, Yt+1]}
   where \emph{j}  \code{is not equal to} \emph{m}.
   \item Vtm: \code{<dbl>}. Value of the uniform intensity of the transition
   from category \emph{m} to all \emph{non-m} categories at time \out{Y<sub>t+1</sub>}
   during time interval \emph{[Yt, Yt+1]}.
   }
 \item transitionStationarity: \code{<tibble>}. A table containing results of
 a stationarity test
 over the loss of \emph{category m} containing 5 columns:
 \enumerate{
   \item To: \code{<fct>}. The gaining category in the transition of interest
   from the category m.
   \item gain: \code{<int>}. Number of times the category had gains from the
   category m.
   \item N: \code{<int>}. Total number of transitions to be considered as stationary (T).
   \item Stationarity: \code{<chr>}. \emph{targeted} or \emph{avoided} the category \code{m}.
   \item Test: \code{<chr>}. \emph{Y} for stationarity detected and \emph{N} when not.
   }
   }

}
}
\section{Slots}{

\describe{
\item{\code{lookupcolor}}{The colors (character vector) associated with the LUC legend items.}

\item{\code{transitionData}}{tibble. A table of Transition level's results (gain n (\emph{Rtin} & \emph{Wtn})
or loss m (\emph{Qtmj} & \emph{Vtm}) values).}

\item{\code{transitionStationarity}}{tibble. A table containing results of a stationarity
test. A change is considered stationary only if the intensities for all time
intervals reside on one side of the uniform intensity, i.e are smaller or bigger
than the uniform rate over the whole period.}
}}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{SL_2002_2014}
\alias{SL_2002_2014}
\title{Tables of land use and cover (LUC) in the São Lourenço River Basin (2002 - 2014)}
\format{
A data list with 5 objects:
\describe{
  \item{lulc_Multistep}{\code{<tibble>} Contingency table for all analysed
  time steps, containing 8 columns:
  \enumerate{
  \item Period: \code{<chr>} The period \emph{[Yt, Yt+1]}.
  \item From: \code{<int>} numerical code of a LUC category \emph{i}.
  \item To: \code{<int>} numerical code of a LUC category \emph{j}.
  \item km2: \code{<dbl>} Area in square kilometers that transited from the
  category \emph{i}
   to category \emph{j} in the period from \emph{Yt} to \emph{Yt+1}.
  \item QtPixel: \code{<int>} Pixel count that transited from the categories
   \emph{i} to category \emph{j} in the period from \emph{Yt} to \emph{Yt+1}.
  \item Interval: \code{<int>} Interval of years between the first and
   the last year of the period \emph{[Yt, Yt+1]}.
  \item yearFrom: \code{<int>} The year that the change comes from \emph{[Yt]}
  \item yearTo: \code{<int>} The year that the change goes for \emph{[Yt+1]}
  }}
  \item{lulc_Onstep}{\code{<tibble>} Contingency table for the entire analysed
   period \emph{[Yt1, YT]}, containing
  8 columns identical with \code{lulc_Mulstistep}}.
  \item{tb_legend}{\code{<tibble>} A table of the pixel value, his name and
  color containing 3 columns:
  \enumerate{
  \item categoryValue: \code{<int>} the pixel value of the LUC category.
  \item categoryName: \code{<fct>} randomly created string associated with a
  given pixel value of a LUC category.
  \item color: \code{<chr>} random color associated with the given pixel value
   of a LUC category.
  }}
  \item{totalArea}{\code{<tibble>} A table with the total area of the study
  area containing 2 columns:
  \enumerate{
  \item area_km2: \code{<dbl>} The total area in square kilometers.
  \item QtPixel: \code{<int>} The total area in pixel counts}}.
  \item{totalInterval}{\code{<int>} Total interval of the analysed time
  series in years}.
  }
}
\source{
Embrapa Pantanal, Instituto SOS Pantanal, WWF-Brasil (2015).
}
\usage{
data(SL_2002_2014)
}
\description{
A list containing five objects created by the \code{\link{contingencyTable}}
function with \code{SaoLourencoBasin} as input
(\code{SL_2002_2014 <- contingenceTable(input_raster = SaoLourencoBasin, pixelresolution = 30)}).
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/OpenLand-package.R
\name{OpenLand-package}
\alias{OpenLand-package}
\alias{OpenLand}
\title{OpenLand: land use and cover (LUC) time series analysis in R.}
\description{
OpenLand is an open-source R package for the analysis of land use and cover
(LUC) time series. It includes support for consistency check and loading
spatiotemporal raster data and synthesized spatial plotting. Several LUC change
(LUCC) metrics in regular or irregular time intervals can be extracted and
visualized through one- and multistep sankey and chord diagrams. A complete
intensity analysis according to \cite{(Aldwaik and Pontius, 2012)} is
implemented, including tools for the generation of standardized multilevel
output graphics.
}
\references{
Aldwaik, S. Z. and Pontius, R. G. (2012) ‘Intensity analysis to unify
measurements of size and stationarity of land changes by interval, category,
and transition, Landscape and Urban Planning. Elsevier B.V., 106(1), pp. 103–114.
\doi{10.1016/j.landurbplan.2012.02.010}.
}
\seealso{
The core functions in this package: \code{\link{intensityAnalysis}},
\code{\link{contingencyTable}},
}
\author{
Reginal Exavier \email{reginalexavier@rocketmail.com}, Peter Zeilhofer \email{zeilhoferpeter@gmail.com}
}
